angular.module('childcareOnApp').controller('CalendarCtrl', ['$scope', '$rootScope', 'CalendarFactory', 'ClassroomFactory', 'ChildFactory', 'CommonFactory', '$state', '$stateParams', 'ChildcareOnUtils', '$translate', '$locale', '$ionicModal', '$q', CalendarCtrl]);

function CalendarCtrl($scope, $rootScope, CalendarFactory, ClassroomFactory, ChildFactory, CommonFactory, $state, $stateParams, ChildcareOnUtils, $translate, $locale, $ionicModal, $q) {

  // https://github.com/twinssbc/Ionic-Calendar

  // switch(idioma) { case 'es_ES': $locale.DATETIME_FORMATS.MONTH=trads.es_ES.MONTH; $locale.DATETIME_FORMATS.SHORTDAY=trads.es_ES.SHORTDAY; break; case 'en_EN': $locale.DATETIME_FORMATS.MONTH=trads.en_GB.MONTH; $locale.DATETIME_FORMATS.SHORTDAY=trads.en_GB.SHORTDAY; break; }
  $locale.DATETIME_FORMATS.MONTH = ChildcareOnUtils.getMonthsTranslated();
  $locale.DATETIME_FORMATS.SHORTDAY = ChildcareOnUtils.getShortDaysTranslated();

  // $locale('es');

  var vm = this;
  vm.calendar = {};
  var now = new Date();
  vm.viewTitle = $translate.instant(ChildcareOnUtils.getMonth(now.getMonth())) + ' ' + now.getFullYear();
  vm.addEventParams = {
    notifications: [60]
  };
  vm.events = {};
  vm.filterEvents = {};
  vm.classroomChildren = [];
  vm.calendarMode = ['month', 'week', 'day'];
  vm.selected = {};
  vm.eventsJoined = [];
  if ($rootScope.allUserInfo.type !== 'users') {
    var typeSelection = [
      {
        name: 'calendar_classroom',
        value: 'classroom'
      },
      {
        name: 'calendar_child',
        value: 'child'
      }
    ];
    if ($rootScope.allUserInfo.type === 'principal') {
      typeSelection.unshift({
        name: 'calendar_allSchool',
        value: 'allSchool'
      });
      // typeSelection.push({
      //   name: 'calendar_type_teachers',
      //   type: 'teachers'
      // });
    }
    vm.typeSelection = typeSelection;
    vm.notificationSelection = [
      {
        name: $translate.instant('calendar_minutes', {value: 10}),
        value: 10
      },
      {
        name: $translate.instant('calendar_minutes', {value: 20}),
        value: 20
      },
      {
        name: $translate.instant('calendar_minutes', {value: 30}),
        value: 30
      },
      {
        name: $translate.instant('calendar_minutes', {value: 40}),
        value: 40
      },
      {
        name: $translate.instant('calendar_minutes', {value: 50}),
        value: 50
      },
      {
        name: $translate.instant('calendar_hour'),
        value: 60
      }
    ];
  }

  function changeMode (mode) {
    vm.calendar.mode = mode;
  }
  function isToday () {
    var todayInput = new Date(),
        currentCalendarDate = new Date(vm.calendar.currentDate);

    todayInput.setHours(0, 0, 0, 0);
    currentCalendarDate.setHours(0, 0, 0, 0);
    return todayInput.getTime() === currentCalendarDate.getTime();
  }
  function today () {
    vm.calendar.currentDate = new Date();
  }
  function onViewTitleChanged (title) {
    console.log('title is', title, vm.calendar.mode);
    vm.viewTitle = title;
    delete vm.addEventParams.day;
    delete vm.addEventParams.hour;
    var currentDateTime = vm.calendar.currentDate.getTime();
    if ((currentDateTime - vm.from.getTime()) < 2505600000 || (vm.to.getTime() - currentDateTime) < 2505600000) {
      vm.from.setMonth(vm.calendar.currentDate.getMonth() -2);
      vm.to.setMonth(vm.calendar.currentDate.getMonth() +3);
      loadData(false);
    }
  }
  function onTimeSelected (selectedTime, events, disabled) {
    console.log('Selected time: ' + selectedTime + ', hasEvents: ' + (events !== undefined && events.length !== 0) + ', disabled: ' + disabled);
    vm.addEventParams.day = new Date(selectedTime.getFullYear(), selectedTime.getMonth(), selectedTime.getDate(), 10, 0, 0);
    vm.addEventParams.hour = vm.addEventParams.day;
  }
  function onEventSelected (event) {
    var filtered = vm.events[event.type].filter(function(element) {
      return element.$id === event.id;
    });
    vm.detailEvent = filtered[0];
    vm.detailEvent.type = event.type;
    vm.modalDetail.show();
  }
  function openCreateEvent() {
    if (!vm.addEventParams.day) {
      var dateToInput = new Date();
      vm.addEventParams.day = new Date(dateToInput.getFullYear(), dateToInput.getMonth(), dateToInput.getDate() +1, 10, 0, 0);
      vm.addEventParams.hour = vm.addEventParams.day;
    }
    vm.modalAdd.show();
  }
  function addEvent() {
    console.log('vm.addEventParams', vm.addEventParams);
    if (!vm.addEventParams.title || vm.addEventParams.title === '' ||
      !vm.addEventParams.type || vm.addEventParams.type === '') {
      return alert($translate.instant('fillFields'));
    }
    $rootScope.show();
    var day = new Date(vm.addEventParams.day);
    var hour = new Date(vm.addEventParams.hour);
    var eventMoment = new Date(day.getFullYear(), day.getMonth(), day.getDate(), hour.getHours(), hour.getMinutes());
    var event = {
      title: vm.addEventParams.title,
      allDay: false,
      date: eventMoment.getTime(),
      notifications: vm.addEventParams.notifications,
    };
    if (!!vm.addEventParams.description && vm.addEventParams.description !== '') {
      event.description = vm.addEventParams.description;
    }
    var ids = [];
    for (var id in vm.addEventParams.who) {
      if (!!vm.addEventParams.who[id]) {
        ids.push(id);
      }
    }
    var notificationTime = new Date(eventMoment);
    notificationTime.setMinutes(notificationTime.getMinutes() - parseInt(vm.addEventParams.notifications[0]));
    var notificationToAdd = {
      date: notificationTime.getTime(),
      type: vm.addEventParams.type,
      title: event.title,
      eventDate: eventMoment.getTime()
    };
    if (ids.length > 0) {
      notificationToAdd.ids = ids;
    }
    CalendarFactory.addNotification(notificationToAdd)
    .then(function(ref) {
      console.log('ref', ref);
      var newNotifications = {};
      if (Array.isArray(ref)) {
        ids.forEach(function(id) {
          newNotifications[id] = {};
        });
        ref.forEach(function(reference, index) {
          newNotifications[ids[index]][reference.key] = vm.addEventParams.notifications[0];
        });
        event.notificationsIds = newNotifications;
      } else {
        newNotifications[ref.key] = vm.addEventParams.notifications[0];
        event.notifications = newNotifications;
      }
      CalendarFactory.addEventToCalendar(vm.addEventParams.type, ids, event)
      .then(function() {
          $rootScope.hide();
          vm.addEventParams = {
            notifications: [60]
          };
          vm.modalAdd.hide();
          alert($translate.instant('createdSuccessfully'));
          parseEvents();
      }, function() {
        // add common factory error
        var obj = { user: $rootScope.allUserInfo.id, error:stripslashes(JSON.stringify(e)), page:$state.current.name, date: JSON.stringify(new Date())};
        CommonFactory.saveLog(obj);
        alert($translate.instant('serverError'));
      });
    }, function() {
      // add common factory error
      var obj = { user: $rootScope.allUserInfo.id, error:stripslashes(JSON.stringify(e)), page:$state.current.name, date: JSON.stringify(new Date())};
      CommonFactory.saveLog(obj);
      alert($translate.instant('serverError'));
    });
  }
  function changeType() {
    vm.addEventParams.who = {};
    delete vm.addEventParams.classroom;
  }
  function loadChildrenList() {
    delete vm.addEventParams.who;
    delete vm.childrenList;
    vm.childrenList = ClassroomFactory.getStudentsClassroom(vm.addEventParams.classroom);
  }
  function filter() {
    if ($rootScope.allUserInfo.type !== 'user') {
      return filterPrincipal();
    } else {
      return filterUser();
    }
  }
  function filterPrincipal() {
    if (!vm.filterEvents.school) {
      delete vm.events.school;
    }
    if (!!vm.filterEvents.classroom) {
      if (!vm.filterEvents.classroomId) {
        return alert($translate.instant('fillFields'));
      }
      vm.events.classroom = CalendarFactory.getCalendar('classroom', vm.filterEvents.classroomId, vm.from, vm.to);
      vm.selected.classroom = $rootScope.school.classrooms[vm.filterEvents.classroomId];
    } else {
      delete vm.filterEvents.classroomId;
      delete vm.events.classroom;
    }
    if (!!vm.filterEvents.child) {
      if (!vm.filterEvents.childId) {
        return alert($translate.instant('fillFields'));
      }
      vm.events.child = CalendarFactory.getCalendar('child', vm.filterEvents.childId, vm.from, vm.to);
      var childrenFiltered = vm.classroomChildren.filter(function (child) {
        return child.$id === vm.filterEvents.childId;
      });
      vm.selected.child = childrenFiltered[0].name;
    } else {
      delete vm.filterEvents.childClassroomId;
      delete vm.filterEvents.childId;
      delete vm.events.child;
    }
    initWatcher();
    vm.modalSettings.hide();
  }
  function filterUser() {
    if (!vm.filterEvents.school) {
      delete vm.events.school;
    }
    if (!!vm.filterEvents.classroom) {
      vm.events.classroom = CalendarFactory.getCalendar('classroom', $rootScope.allUserChilds[$rootScope.childSelected].classroomId, vm.from, vm.to);
      vm.selected.classroom = $rootScope.allUserChilds[$rootScope.childSelected].classroomName;
    } else {
      delete vm.filterEvents.classroomId;
      delete vm.events.classroom;
    }
    if (!!vm.filterEvents.child) {
      vm.events.child = CalendarFactory.getCalendar('child', $rootScope.childSelected, vm.from, vm.to);
      vm.selected.child = $rootScope.allUserChilds[$rootScope.childSelected].name;
    } else {
      delete vm.filterEvents.childClassroomId;
      delete vm.filterEvents.childId;
      delete vm.events.child;
    }
    initWatcher();
    vm.modalSettings.hide();
  }
  function getClassroomChildren() {
    vm.classroomChildren = ChildFactory.getChildrenFromClassroom(vm.filterEvents.childClassroomId);
  }
  function deleteEvent() {
    _confirm($translate.instant('confirmRemoveGeneric'), $translate.instant('confirmGeneric') ,$translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_accept'), function(buttonIndex){
      if(buttonIndex === 2){
        var promises = [];
        if (vm.detailEvent.notificationsIds) {
          for (var classroomId in vm.detailEvent.notifications) {
            for (var notificationId in vm.detailEvent.notifications[classroomId]) {
              console.log('notificationId', notificationId);
              promises.push(
                CalendarFactory.deleteNotification(notificationId)
              );
            }
          }
        } else if (!!vm.detailEvent.notifications) {
          for (var notificationId in vm.detailEvent.notifications) {
            console.log('notificationId', notificationId);
            promises.push(
              CalendarFactory.deleteNotification(notificationId)
            );
          }
        }
        var typeId;
        switch (vm.detailEvent.type) {
          case 'allSchool': typeId = $rootScope.allUserInfo.school; break;
          case 'classroom': typeId = vm.filterEvents.classroomId || $rootScope.classSelected.id;
          break;
          case 'child': typeId = vm.filterEvents.childId; break;
        }
        promises.push(
          CalendarFactory.deleteEvent(vm.detailEvent.$id, vm.detailEvent.type, typeId)
        );
        $q.all(promises).then(function() {
          $rootScope.showToast('infoDeletedSuccessfully');
          vm.modalDetail.hide();
          parseEvents();
        }, function() {
          alert($translate.instant('serverError'));
        });
      }
    });
  }
  function getFirstNotification() {
    var key = Object.keys(vm.detailEvent.notifications)[0];
    return vm.detailEvent.notifications[key];
  }

  function parseEvents() {
    var eventsJoined = [];
    for (var key in vm.events) {
      vm.events[key].forEach(function(event) {
        eventsJoined.push({
          title: event.title,
          allDay: false,
          startTime: new Date(event.date),
          endTime: new Date(event.date + 3600000),
          type: key,
          id: event.$id
        });
      });
    }
    vm.eventsJoined = eventsJoined;
    console.log('eventsJoined', eventsJoined);
  }

  function loadData(showLoading) {
    switch($rootScope.allUserInfo.type) {
      case 'principal': vm.events = {
        allSchool: CalendarFactory.getCalendar('allSchool', $rootScope.allUserInfo.school, vm.from, vm.to),
        classroom: CalendarFactory.getCalendar('classroom', $rootScope.classSelected.id, vm.from, vm.to)
      };
      vm.filterEvents = {
        allSchool: true,
        classroom: true
      };
      vm.selected = {
        classroom: $rootScope.classSelected.name
      };
      break;
      case 'teacher': vm.events = {
        allSchool: CalendarFactory.getCalendar('allSchool', $rootScope.allUserInfo.school, vm.from, vm.to),
        classroom: CalendarFactory.getCalendar('classroom', $rootScope.classSelected.id, vm.from, vm.to)
      };
      vm.filterEvents = {
        allSchool: true,
        classroom: true
      };
      vm.selected = {
        classroom: $rootScope.classSelected.name
      };
      break;
      case 'user': vm.events = {
        allSchool: CalendarFactory.getCalendar('allSchool', $rootScope.allUserChilds[$rootScope.childSelected].school, vm.from, vm.to),
        classroom: CalendarFactory.getCalendar('classroom', $rootScope.allUserChilds[$rootScope.childSelected].classroomId, vm.from, vm.to),
        child: CalendarFactory.getCalendar('child', $rootScope.childSelected, vm.from, vm.to)
      };
      vm.filterEvents = {
        allSchool: true,
        classroom: true,
        child: true
      };
      vm.selected = {
        classroom: $rootScope.allUserChilds[$rootScope.childSelected].classroomName,
        child: $rootScope.allUserChilds[$rootScope.childSelected].name
      };
      break;
    }
    if (!!showLoading) {
      $rootScope.show();
    }
    initWatcher();
  }

  function initWatcher() {
    var promises = [];
    console.log('vm.events', vm.events);
    for (var key in vm.events) {
      promises.push(vm.events[key].$loaded());
    }
    $q.all(promises).then(function() {
      $rootScope.hide();
      parseEvents();
      // vm.events.$watch(function(event){
      //   console.log('trigger');
      //   parseEvents();
      // });
      $scope.$watch('vm.events', function() {
        console.log('trigger');
        parseEvents();
      });
      console.log('vm.events', vm.events);
    });
  }

  vm.changeMode = changeMode;
  vm.isToday = isToday;
  vm.today = today;
  vm.onViewTitleChanged = onViewTitleChanged;
  vm.onTimeSelected = onTimeSelected;
  vm.openCreateEvent = openCreateEvent;
  vm.addEvent = addEvent;
  vm.deleteEvent = deleteEvent;
  vm.changeType = changeType;
  vm.loadChildrenList = loadChildrenList;
  vm.filter = filter;
  vm.getClassroomChildren = getClassroomChildren;
  vm.onEventSelected = onEventSelected;
  vm.getFirstNotification = getFirstNotification;

  $scope.$on('$destroy', function() {
    if (!!vm.modalAdd) {
      vm.modalAdd.remove();
    }
    if (!!vm.modalSettings) {
      vm.modalSettings.remove();
    }
    if (!!vm.modalDetail) {
      vm.modalDetail.remove();
    }
  });

  $ionicModal.fromTemplateUrl('modal-add.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    vm.modalAdd = modal;
  });
  $ionicModal.fromTemplateUrl('modal-settings.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    vm.modalSettings = modal;
  });
  $ionicModal.fromTemplateUrl('modal-detail.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    vm.modalDetail = modal;
  });

  (function(){
    var from = new Date();
    from.setDate(1);
    from.setHours(0);
    from.setMinutes(0);
    from.setMonth(from.getMonth() -3);
    var to = new Date();
    to.setMonth(to.getMonth() +4);
    vm.from = from;
    vm.to = to;
    loadData(true);
  }());
}