angular.module('childcareOnApp').controller('CreateCommunicationCtrl', function($scope, $rootScope, $translate, $stateParams, CommunicationsFactory, ImageFactory, FirebasePluginFactory, $ionicHistory, $ionicModal, $timeout, $q) {
  $rootScope.addSomething = false;
  $scope.communicationMessage = {};
  $scope.docDisabled = false;
  $scope.docRoute = "";
  $scope.progress = {};
  $scope.uploading = {};

  var uploadDocument = function(file){
    return $q(function(resolve,reject){
      if(typeof file === 'undefined'){
        reject('undefined');
        return;
      }
      if(file.indexOf('application/pdf') === -1){
        alert($translate.instant('communications_noPdf'));
        reject('noPdf');
      }
      file = file.split('base64,');
      file = file[file.length -1];
      $scope.$evalAsync(function () {
        $scope.uploading.document = true;
        $scope.progress.document = 0;
      });

      var interval = setInterval(function() {
        $scope.$evalAsync(function () {
          $scope.progress.document = ImageFactory.getPercentageProgress();
        });
      }, 200);

      ImageFactory.uploadCommunicationImage(file,"application/pdf",false).then(function(url){
        $scope.docLink = url;
        delete $rootScope.documentUrl;
        clearInterval(interval);
        resolve();
      },function(){
        clearInterval(interval);
        reject('error');
      });
    });
  };

  $scope.sendCommunication = function(){
    console.log($scope.communicationMessage);
    if(!$scope.communicationMessage.title || !$scope.communicationMessage.message){
      return alert($translate.instant('fillFields'));
    }
    $rootScope.show();
    var time = ($scope.communicationMessage.whenSend.value === 'scheduled')? $scope.communicationMessage.whenSend.time : null;
    if ($scope.type === 'classrooms') {
      CommunicationsFactory.sendMessage($scope.classroomsToSend, $scope.communicationMessage, $scope.imageLink,$scope.docLink, $scope.communicationMessage.addedLink1, $scope.communicationMessage.addedLink2, time).then(function(messageId) {
        CommunicationsFactory.saveReadedMessage(messageId);
        FirebasePluginFactory.logEvent('send_communication');
        $rootScope.hide();
        init();
        alert($translate.instant('communications_communicationSent'));
        $ionicHistory.goBack();
      },function(){
        alert($translate.instant('serverError'));
      });
    } else {
      for (var childId in $scope.childrenToSend) {
        if (!$scope.childrenToSend[childId]) {
          delete $scope.childrenToSend[childId];
        }
      }
      if (Object.keys($scope.childrenToSend).length === 0) {
        $rootScope.hide();
        return alert($translate.instant('select_atLeast_one_child'));
      }
      CommunicationsFactory.sendIndividualMessage($scope.childrenToSend, $scope.communicationMessage, $scope.imageLink,$scope.docLink, $scope.communicationMessage.addedLink1, $scope.communicationMessage.addedLink2, time).then(function(messageId) {
        CommunicationsFactory.saveReadedMessage(messageId);
        $rootScope.hide();
        init();
        alert($translate.instant('communications_communicationSent'));
        $ionicHistory.goBack();
      },function(){
        alert($translate.instant('serverError'));
      }); 
    }
    
  };

  $scope.uploadImage = function(){
    if(typeof $scope.communicationMessage.images === 'object' && $scope.communicationMessage.images.length === 4){
      return alert($translate.instant('communications_maximumImages'));
    }
    // if($scope.imageDisabled){
    //   alert($translate.instant('communications_alreadyUploaded'));
    //   return;
    // }
    $translate(['communications_selectImage','communications_imageTitle','communications_camera','communications_gallery','login_forgotPassword_cancelButton']).then(function(t){
      navigator.notification.confirm(
        t.communications_selectImage, // message
         function(index){
           if(index === 2) return;
           var source = (index === 1)? Camera.PictureSourceType.CAMERA: Camera.PictureSourceType.PHOTOLIBRARY;
           navigator.camera.getPicture(onSuccess, onFail, { correctOrientation:true,quality: 80,targetWidth: 1080, targetHeight:1020, sourceType: source,
             destinationType: Camera.DestinationType.DATA_URL, encodingType: Camera.EncodingType.JPEG
           });
         },
        t.communications_imageTitle,           // title
        [t.communications_camera, t.login_forgotPassword_cancelButton, t.communications_gallery]     // buttonLabels
      );
    });
  };

  $scope.uploadImages = function() {
    if(typeof $scope.communicationMessage.images === 'object' && $scope.communicationMessage.images.length === 4){
      return alert($translate.instant('communications_maximumImages'));
    }
    var left = 4;
    if (!!$scope.communicationMessage.images) {
      left = left - $scope.communicationMessage.images.length;
    }
    var args = {
      'selectMode': 100, //101=picker image and video , 100=image , 102=video
      'maxSelectCount': left, //default 40 (Optional)
      'maxSelectSize': 188743680, //188743680=180M (Optional)
    };
    MediaPicker.getMedias(args, function(medias) {
      //medias [{mediaType: "image", path:'/storage/emulated/0/DCIM/Camera/20170808_145202.jpg', size: 21993}]
      if (!medias || medias.length === 0) {
        return console.log('no images selected');
      }
      ImageFactory.multipleImages.getCompressedImages(medias)
      .then(function(thumbnails) {
        ImageFactory.multipleImages.uploadImages(thumbnails, 'communications/' + $rootScope.allUserInfo.school)
        .then(function (imagesUploaded) {
          $rootScope.showToast("imageUpdatedSuccess");
          if(!$scope.communicationMessage.images) $scope.communicationMessage.images = [];
          for (var i=0; i<imagesUploaded.length; i++) {
            $scope.communicationMessage.images.push({image: imagesUploaded[i].image});
          }
        });
      });
    });
  };

  $scope.finishUpload = function(file) {
    if (typeof file === 'undefined' || !file) {
      return false;
    }
    console.log("finish upload");
    uploadDocument(file).then(function(){
      $scope.$evalAsync(function() {
        $scope.docDisabled = true;
        $scope.uploading.document = false;
      });
    },function(error){
      if(error === 'error') alert($translate.instant('serverError'));
    });
  };

  $scope.deleteAttachedDoc = function() {
    _confirm($translate.instant('confirmRemoveGeneric'), $translate.instant('confirmGeneric') ,$translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_accept'), function(buttonIndex){
      if(buttonIndex === 2){
        $scope.$evalAsync(function() {
          $scope.docDisabled = false;
          $scope.uploading.document = false;
          delete $scope.docLink;
        });
      }
    });
  };

  function onSuccess(imageData) {
    console.log("success",imageData);
    ImageFactory.uploadCommunicationImage(imageData,"image/jpeg").then(function(url){
      console.log("imagen subida",url);
      $scope.imageLink = url;
      $rootScope.showToast("imageUpdatedSuccess");
      $scope.imageDisabled = true;
      if(!$scope.communicationMessage.images) $scope.communicationMessage.images = [];
      $scope.communicationMessage.images.push({image:url});
    },function(){
      alert($translate.instant('serverError'));
    });
  }

  function onFail(message) {
    if(message.indexOf('cancelled') === -1) alert($translate.instant('imageUpdatedError'));
    console.log('Failed because: ' + message);
  }

  $scope.$on("$ionicView.afterEnter", function(){
    $scope.type = $stateParams.type;
    $scope.imageDisabled = false;
    $scope.childrenToSend = {};
    $scope.classroomsToSend = {};
    $scope.classroomsToSend[$rootScope.classSelected.id] = true;
    $scope.classroomsToSendName = angular.copy($rootScope.classSelected);
  });

  function setCurrentDate() {
    var now = new Date();
    now.setMilliseconds(0);
    now.setSeconds(0);
    return now;
  }

  $ionicModal.fromTemplateUrl('modal-schedule.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });
  $scope.openModal = function() {
    $scope.communicationMessage.whenSend.value = 'scheduled';
    $scope.communicationMessage.whenSend.time = setCurrentDate();
    $scope.modal.show();
  };
  $scope.cancelModal = function() {
    $scope.communicationMessage.whenSend.value = 'now';
    delete $scope.communicationMessage.whenSend.time;
    $scope.modal.hide();
  };
  $scope.sendModal = function() {
    if(!$scope.communicationMessage.whenSend.time) return alert($translate.instant('fillFields'));
    if($scope.communicationMessage.whenSend.time.getTime() < new Date().getTime()) return alert($translate.instant('communications_datePrevious'));
    $scope.sendCommunication();
    $scope.modal.hide();
  };
  // Cleanup the modal when we're done with it!
  $scope.$on('$destroy', function() {
    if (!!$scope.modal) {
      $scope.modal.remove();
    }
  });

  function init(){
    $scope.communicationMessage = {
      whenSend:{value:'now'}
    };
  }

  (function(){
    init();
  }());
});
