angular.module('childcareOnApp').directive('parentsDocuments', function() {
  return {
    restrict: 'E', //attribute or element
    transclude: true,
    scope: {
      parentsDocuments: '=',
      studentId: '=',
      isToday: '=',
    },
    templateUrl: 'app/components/diary/parents-documents/parents-documents.html',
    controller: ['$scope','$rootScope', '$q', '$translate', 'DiaryFactory', 'ImageFactory', '$state', '$ionicActionSheet', function($scope, $rootScope, $q, $translate, DiaryFactory, ImageFactory, $state, $ionicActionSheet) {

      var toUpload = {}, textToUpload = false;
      console.log('parentsDocuments', $scope.parentsDocuments);
      $scope.uploading = false;
      $scope.evidenceDocument = {};

      function askPermissions() {
        return $q(function(resolve, reject) {
          ImageFactory.getAndAskStoragePermissions('read')
          .then(function() {
            ImageFactory.getAndAskStoragePermissions('write')
            .then(function() {
              resolve();
            }, function() {
              resolve();
            });
          }, function() {
            resolve();
          });
        });
      }

      function upload() {
        toUpload.text = textToUpload;
        $scope.parentsDocuments.$add(toUpload).then(function () {
          $scope.$evalAsync(function () {
            $scope.uploading = false;
          });
          console.log('toUpload', toUpload);
          if (toUpload.type === 'image') {
            alert($translate.instant('imageUpdatedSuccess'));
          } else {
            alert($translate.instant('home_notifications_documentUploaded'));
          }
          toUpload = {};
          textToUpload = false;
        });
      }

      function textPrompt() {
        navigator.notification.prompt(
          $translate.instant('diary_parentsDocuments_inputText'),  // message
          function (results) {
            if(results.buttonIndex === 1 && results.input1 !== ''){
              textToUpload = results.input1;
            } else {
              textToUpload = '';
            }
            if (!!toUpload.type) {
              upload();
            }
          },                  // callback to invoke
          $translate.instant('diary_uploading'),            // title
          [$translate.instant('button_send'),$translate.instant('login_forgotPassword_cancelButton')],             // buttonLabels
          ''                 // defaultText
        );
      }

      $scope.uploadPhotos = function() {
        askPermissions()
        .then(function() {
          var args = {
            'selectMode': 100, //101=picker image and video , 100=image , 102=video
            'maxSelectCount': 5, //default 40 (Optional)
            'maxSelectSize': 188743680, //188743680=180M (Optional)
          };
          MediaPicker.getMedias(args, function(medias) {
            //medias [{mediaType: "image", path:'/storage/emulated/0/DCIM/Camera/20170808_145202.jpg', size: 21993}]
            if (!medias || medias.length === 0) {
              return console.log('no images selected');
            }
            textPrompt();
            $scope.$evalAsync(function () {
              $scope.uploading = true;
            });
            ImageFactory.multipleImages.getCompressedImages(medias)
            .then(function(thumbnails) {
              ImageFactory.multipleImages.uploadImages(thumbnails, 'diary/' + $scope.studentId + '/parentsDocuments')
              .then(function (imagesUploaded) {
                toUpload = {
                  images: imagesUploaded,
                  type: 'image',
                  date: new Date().getTime()
                };
                if (!!textToUpload || textToUpload === '') {
                  upload();
                }
              });
            });
          }, function(e) { 
            console.log(e);
            alert($translate.instant('serverError'));
          });
        });
      };

      $scope.uploadVideos = function() {
        askPermissions()
        .then(function() {
          var args = {
            'selectMode': 102, //101=picker image and video , 100=image , 102=video
            'maxSelectCount': 5, //default 40 (Optional)
            'maxSelectSize': 188743680, //188743680=180M (Optional)
          };
          console.log('before MediaPicker');
          MediaPicker.getMedias(args, function(medias) {
            if (!medias || medias.length === 0) {
              return console.log('no images selected');
            }
            console.log('videos received', medias);
            textPrompt();
            $scope.$evalAsync(function () {
              $scope.uploading = true;
            });
            ImageFactory.multipleImages.getCompressedVideos(medias)
            .then(function(thumbnails) {
              ImageFactory.multipleImages.uploadVideos(thumbnails, 'diary/' + $scope.studentId + '/parentsDocuments')
              .then(function (videosUploaded) {
                videosUploaded = videosUploaded.map(function(video) {
                  delete video.month;
                  return video;
                });
                console.log('videosUploaded', videosUploaded);
                toUpload = {
                  videos: videosUploaded,
                  type: 'video',
                  date: new Date().getTime()
                };
                if (!!textToUpload || textToUpload === '') {
                  upload();
                }
              });
            });
          }, function(e) { 
            console.log(e);
            alert($translate.instant('serverError'));
          });
        });
      };

      $scope.finishUpload = function(file) {
        if (typeof file === 'undefined' || !file) {
          return false;
        }
        var type = file.type || '';
        if (!!type && type.indexOf('image') !== -1) {
          alert($translate.instant('diary_wrongDocumentFormat_image'));
          return;
        }
        if (!!type && type.indexOf('video') !== -1) {
          alert($translate.instant('diary_wrongDocumentFormat_video'));
          return;
        }
        $scope.$evalAsync(function () {
          $scope.uploading = true;
        });
        textPrompt();
        ImageFactory.saveImageFromBlob(file, 'diary/' + $scope.studentId + '/parentsDocuments', new Date().getTime().toString(), file.type, false)
        .then(function (url) {
          toUpload = {
            date: new Date().getTime(),
            type: 'document',
            url: url,
            fileType: file.type
          };
          if (!!textToUpload || textToUpload === '') {
            upload();
          }
        });
      };

      $scope.selectPhoto = function(image) {
        $state.go('app.imageViewer',{img:image.image, canDelete:false, imageComplete: image, canDownload: false});
      };

      $scope.selectDocument = function(item) {
        console.log('item is', item);
        if (item.fileType !== 'application/pdf') {
          return window.open(item.url, '_system', 'location=false');
        }
        var doc = {
          name: item.date.toString(),
          url: item.url
        };
        $rootScope.show();
        ImageFactory.openDocument(doc)
        .then(function() {
          $rootScope.hide();
        }, function() {
          $rootScope.hide();
          window.open(item.url, '_system', 'location=yes');
        });
      };

      $scope.markAsChecked = function(item) {
        $scope.parentsDocuments.$save(item)
        .then(function() {
          $rootScope.showToast('infoSavedSuccessfully');
        }, function (error) {
          console.log('error', error);
          alert($translate.instant('serverError'));
        });
      }

      $scope.clickMore = function(item) {
        var ionicActionSheet;
        var options = {
          buttons: [
            { text: '<i class="icon ion-eye"></i>'+ $translate.instant('button_see') }
          ],
          titleText: item.text,
          cancelText: $translate.instant('login_forgotPassword_cancelButton'),
          cancel: function() {
            // add cancel code..
          },
          buttonClicked: function(index) {
            console.log('button clicked', index);
            switch(index) {
              case 0: window.open(item.url, '_system', 'location=false'); break;
              default: console.log('default');
            }
            ionicActionSheet();
            return true;
          }
        };
        if ($rootScope.allUserInfo.type === 'user') {
          options.destructiveText = '<i class="icon ion-trash-a"></i>'+ $translate.instant('documents_delete');
          options.destructiveButtonClicked = function() {
            $scope.parentsDocuments.$remove(item)
            .then(function() {
              $rootScope.showToast('infoDeletedSuccessfully');
            });
            ionicActionSheet();
          }
        }
        ionicActionSheet = $ionicActionSheet.show(options);
      };

    }],
    link: function($scope, elem, attr, ctrl) {
    }
  };
})