angular.module('childcareOnApp').controller('GalleryCtrl', function($scope, $rootScope, $translate, $filter, ImageFactory, ionicMaterialInk, ionicMaterialMotion, $state, $ionicActionSheet, configuration) {

  ionicMaterialInk.displayEffect();
  ionicMaterialMotion.ripple();
  var classroomId, schoolId, loadedClassroomId;
  $scope.folders = [];
  $scope.maxImages = 60;

  $scope.getIfSettingsValidate = function(){
    if(!$rootScope.settings.hasOwnProperty('photo') || !$rootScope.settings.photo.hasOwnProperty('type')) return false;
    return ($rootScope.settings.photo.type === 'validate');
  };

  $scope.selectPhoto = function(image){
    var imagesInfo = angular.copy($rootScope.imagesInfo);
    imagesInfo = $filter('todayPhotos')(imagesInfo);
    console.log('imagesInfo', imagesInfo);
    var index = 0;
    imagesInfo.forEach(function(imageArray, i) {
      if (imageArray.location === image.location) {
        index = i;
      }
    });
    $state.go('app.multiImage',{img: imagesInfo, index: index, canDelete: true, imageComplete: image, canDownload: (!!$rootScope.settings && !!$rootScope.settings.premium && !!$rootScope.settings.downloadGallery)});
    // $state.go('app.imageViewer',{img:image.image, canDelete:true, imageComplete: image, canDownload: !!$rootScope.settings.downloadGallery});
  };

  function getImages(classroomId){
    loadedClassroomId = classroomId;
    $rootScope.imagesInfo = ImageFactory.getDatabaseImages(classroomId, $scope.maxImages);
    $rootScope.imagesInfo.$loaded()
    .then(function() {
      getFolders();
      setTimeout($rootScope.hide, 100);
    }, function() {
      console.log("error en getImages");
      alert($translate.instant('serverError'));
    });
  };

  $scope.loadMoreImages = function() {
    $rootScope.show();
    if ($scope.maxImages === 60) {
      $scope.maxImages = 150;
    } else {
      $scope.maxImages = 'full';
    }
    getImages(loadedClassroomId);
  };

  $scope.deleteImageHelper = function(item){
    if($rootScope.allUserInfo.type === 'user') return;
    $translate(['gallery_deleteImageText','gallery_deleteImageTitle','login_forgotPassword_cancelButton','yes']).then(function(t){
      _confirm(t.gallery_deleteImageTitle,t.gallery_deleteImageText,t.login_forgotPassword_cancelButton,t.yes,
        function(buttonIndex){
        if(buttonIndex === 2) deleteImage(item);
      });
    });
  };

  $scope.goToPhotos = function(folder){
    // console.log('folder', folder);
    $state.go('app.photos',{monthDate:folder});
  };

  $scope.goToPhotosCustom = function(folder) {
    // console.log('folder', folder);
    $state.go('app.photos',{customFolder:folder});
  }

  var deleteImage = function(item){
    if(!$rootScope.imagesInfo){
      alert($translate.instant('contentNotLoaded'));
      $rootScope.show();
      return;
    }
    var index = $rootScope.imagesInfo.indexOf(item);
    $rootScope.imagesInfo.$remove(index).then(function(){
      $rootScope.showToast('infoDeletedSuccessfully');
    },function(){
      alert($translate.instant('serverError'));
    });
  };

  var getFolders = function(){
    var folders = {};
    for(var i=$rootScope.imagesInfo.length -1; i>=0; i--){
      if($rootScope.imagesInfo[i].hide || !$rootScope.imagesInfo[i].date) continue;
      var date = new Date(parseInt($rootScope.imagesInfo[i].date+"000"));
      if(!folders.hasOwnProperty(date.getMonth()+"/"+date.getFullYear())) {
        folders[date.getMonth()+"/"+date.getFullYear()] = {month:date.getMonth(), year:date.getFullYear(), photos:[]};
      }
      folders[date.getMonth()+"/"+date.getFullYear()].photos.push($rootScope.imagesInfo[i]);
    }
    var foldersArr = [];
    for(var i in folders){
      foldersArr.push(folders[i]);
    }
    // console.log("foldersArr",foldersArr);
    $scope.folders = foldersArr;
  };

  $scope.holdImage = function(image) {
    console.log('image holded!');
    var options = {
      buttons: [
        { text: '<i class="icon ion-eye"></i>'+ $translate.instant('diary_seeImage') },
        { text: '<i class="icon ion-android-download"></i>'+ $translate.instant('download_image') }
      ],
      titleText: $translate.instant('communications_images'),
      cancelText: $translate.instant('login_forgotPassword_cancelButton'),
      cancel: function() {
        // add cancel code..
      },
      buttonClicked: function(index) {
        console.log('button clicked', index);
        switch(index) {
          case 0: $scope.selectPhoto(image); break;
          case 1: $scope.downloadImage(image); break;
          default: console.log('default');
        }
        return true;
      }
    };
    if ($rootScope.allUserInfo.type !== 'user') {
      options.destructiveText = '<i class="icon ion-trash-a"></i>'+ $translate.instant('gallery_deleteImageTitle');
      options.destructiveButtonClicked = function() {
        $scope.deleteImageHelper(image);
      }
    }
    $ionicActionSheet.show(options);
  };

  $scope.downloadImage = function(imageToDownload){
    $rootScope.showToast('image_downloading');
    ImageFactory.downloadImgToGallery(imageToDownload.image, configuration.appName + " Img", imageToDownload.date);
  };

  $scope.goToGalleryFolders = function() {
    $state.go('app.galleryFolders');
  };

  $scope.$on("$ionicView.afterEnter", function(){
    if($rootScope.allUserInfo.type === 'user') {
      classroomId = $rootScope.allUserChilds[$rootScope.childSelected].classroomId;
      schoolId = $rootScope.allUserChilds[$rootScope.childSelected].school;
    } else {
      classroomId = $rootScope.classSelected.id;
      schoolId = $rootScope.allUserInfo.school;
    }
    $scope.classroomId = classroomId;
    if(!$rootScope.imagesInfo || loadedClassroomId !== classroomId){
      $rootScope.show();
      getImages(classroomId);
    }
    $scope.foldersCustom = ImageFactory.getCustomFolders(schoolId);
  });

  (function(){
  }());
})
