angular.module('childcareOnApp').controller('GalleryNewImageCtrl', function($scope, $rootScope, $stateParams, $translate, ImageFactory, $ionicHistory, $q, FirebasePluginFactory, CommonFactory) {

  var classroomId, returnObj, mediaType, isVideoCompressed, pendingUploadVideo, hasToCompile = false, avoidCompress = false;
  var progress = 0;

  function startUploadVideo() {
    ImageFactory.getVideoSize($scope.imageSelected.image)
    .then(function(fileInformation) {
      var maxWidth = 1080, maxHeight = 720, maxVideoBitrate = 6000000;
      var width = 600, height = 600, videoBitrate = 6000000;
      if (!!fileInformation.size && fileInformation.type === 'video/mp4' && fileInformation.size < 20000) {
        console.log('upload without compressing');
        isVideoCompressed = true;
        if (pendingUploadVideo) {
          uploadGalleryVideo();
        }
        return;
      }
      if (!!fileInformation.size) {        
        if (fileInformation.size < 20000) {
          width = fileInformation.width;
          height = fileInformation.height;
          videoBitrate = fileInformation.bitrate;
        }
        try {
          if (fileInformation.bitrate > maxVideoBitrate) {
            videoBitrate = maxVideoBitrate;
          } else {
            videoBitrate = fileInformation.bitrate;
          }
          if (fileInformation.width > maxWidth) {
            width = maxWidth;
          } else {
            width = fileInformation.width;
          }
          if (fileInformation.height > maxHeight) {
            height = maxHeight;
          } else {
            height = fileInformation.height;
          }
        } catch (error) {
          console.log('error calculating', error);
        }
      }
      console.log(width, height, videoBitrate);
      VideoEditor.transcodeVideo(
        function (info) {
          console.log("success transcoding video", info);
          $scope.imageSelected.image = info;
          console.log("imageSelected",$scope.imageSelected);
          isVideoCompressed = true;
          if (pendingUploadVideo) {
            uploadGalleryVideo();
          }
        }, // success cb
        function (error) {
          alert($translate.instant('gallery_errorVideo'));
          console.log("error transcoding video", error);
          try {
            CommonFactory.saveLog({
              user: $rootScope.allUserInfo.id,
              error: "error transcoding video. video url: file://" + $scope.imageSelected.image,
              errorMessage: JSON.stringify(error)
            });
          } catch (error) {
            console.log('error saving log', error);
          }
          isVideoCompressed = true;
          if (pendingUploadVideo) {
            uploadGalleryVideo();
          }
        }, // error cb
        {
          fileUri: 'file://' + $scope.imageSelected.image, // the path to the video on the device
          outputFileName: 'childcareon-thumb_'+ new Date().getTime(), // the file name for the transcoded video
          outputFileType: VideoEditorOptions.OutputFileType.MPEG4, // android is always mp4
          optimizeForNetworkUse: VideoEditorOptions.OptimizeForNetworkUse.YES, // ios only
          saveToLibrary: false, // optional, defaults to true
          deleteInputFile: false, // optional (android only), defaults to false
          maintainAspectRatio: true, // optional (ios only), defaults to true
          width: width, // optional, see note below on width and height
          height: height, // optional, see notes below on width and height
          videoBitrate: videoBitrate, // optional, bitrate in bits, defaults to 1 megabit (1000000)
          fps: 24, // optional (android only), defaults to 24
          // audioChannels: 2, // optional (ios only), number of audio channels, defaults to 2
          // audioSampleRate: 44100, // optional (ios only), sample rate for the audio, defaults to 44100
          // audioBitrate: 128000, // optional (ios only), audio bitrate for the video in bits, defaults to 128 kilobits (128000)
          progress: function(info) {
            console.log("progress", info);
            progress = info;
          } // info will be a number from 0 to 100
        }
      );
    });
  }

  function startMultiplePhotos() {
    console.log('thumbnails', $scope.imageSelected.image);
    uploadImages()
    .then(function() {
      $rootScope.showToast('imageUpdatedSuccess');
      $scope.imageSelected = {
        customFolders: {},
        folderType: 'classrooms'
      };
      FirebasePluginFactory.logEvent('uploadPhotoVideo',{method:'multiple', type: 'photo'});
    });
    alert($translate.instant('gallery_uploadingMultipleImages'));
    $ionicHistory.goBack();
  }

  function uploadImages() {
    return $q(function (resolve, reject) {
      var promises = [];
      for (var i=0; i < $scope.imageSelected.image.length; i++) {
        promises.push(uploadImage($scope.imageSelected.image[i]));
      }
      $q.all(promises)
      .then(function (result) {
        resolve();
      });
    });
  }

  function uploadImage(thumb, isFirst) {
    return $q(function (resolve, reject) {
      var location, imagesInfo, idToUpload;
      if ($scope.imageSelected.folderType === 'custom') {
        idToUpload = Object.keys($scope.imageSelected.customFolders)[0];
        location = "galleryFolder/"+ $rootScope.allUserInfo.school + '/' + idToUpload;
        imagesInfo = ImageFactory.getCustomFolderImages(idToUpload, 1);
        delete $scope.imageSelected.classroomsToSend;
      } else {
        location = 'gallery/' + classroomId;
        idToUpload = classroomId;
        imagesInfo = $rootScope.imagesInfo;
        delete $scope.imageSelected.customFolders;
      }
      var date = new Date().getTime().toString();
      date = date.slice(0, date.length - 3);
      date = parseInt(date, 10);
      date = date + thumb.index;
      date = date.toString();
      ImageFactory.saveImageFromBlob(thumb.file, location, date, 'image/jpeg', false)
      .then(function (url) {
        var obj = {
          date: date,
          image: url,
          month: new Date().getMonth(),
          location: location + '/' + date + '.jpg'
        };
        if (isFirst) {
          obj.push = true;
        }

        console.log($scope.imageSelected);
        if ($scope.imageSelected.message && $scope.imageSelected.message !== "") {
          obj.message = $scope.imageSelected.message;
        }
        var principalImage = angular.copy(obj);
        if (!!$scope.imageSelected.classroomsToSend) {
          principalImage.classroomsToSend = $scope.imageSelected.classroomsToSend;
        }
        if (!!$scope.imageSelected.customFolders) {
          principalImage.customFolders = $scope.imageSelected.customFolders;
        }
        console.log("principalImage",principalImage);
        console.log("obj",obj);
        imagesInfo.$add(principalImage).then(function(ref){
          var refId = ref.key;
          var promises = [];
          if ($scope.imageSelected.folderType === 'custom') {
            for(var folderId in $scope.imageSelected.customFolders){
              if(!$scope.imageSelected.customFolders[folderId]) continue;
              promises.push(
                firebase.database().ref('galleryFolders/images').child($rootScope.allUserInfo.school).child(folderId).child(refId).set(obj)
              );
            }
          } else {
            for(var classroomId in $scope.imageSelected.classroomsToSend){
              if(!$scope.imageSelected.classroomsToSend[classroomId] || $rootScope.classSelected.id === classroomId) continue;
              promises.push(
                firebase.database().ref('gallery').child(classroomId).child(refId).set(obj)
              );
            }
          }
          $q.all(promises)
          .then(function (result) {
            console.log('after promises');
            resolve();
          }, function(error) {
            console.log('error', error);
            reject();
          });
        },function(e){
          console.log("failed add",e);
          alert($translate.instant('serverError'));
        });
      });
    });
  }

  function uploadGalleryImage () {
    var fn, imagesInfo, idToUpload;
    if ($scope.imageSelected.folderType === 'custom') {
      fn = ImageFactory.uploadGalleryImageCustomFolder;
      idToUpload = Object.keys($scope.imageSelected.customFolders)[0];
      imagesInfo = ImageFactory.getCustomFolderImages(idToUpload, 1);
      delete $scope.imageSelected.classroomsToSend;
    } else {
      fn = ImageFactory.uploadGalleryImage;
      idToUpload = classroomId;
      imagesInfo = $rootScope.imagesInfo;
      delete $scope.imageSelected.customFolders;
    }
    fn($scope.imageSelected.image, idToUpload, mediaType).then(function(obj){
      returnObj = obj;
      returnObj.push = true;
      console.log($scope.imageSelected);
      if ($scope.imageSelected.message && $scope.imageSelected.message !== "") {
        returnObj.message = $scope.imageSelected.message;
      }
      var principalImage = angular.copy(returnObj);
      if (!!$scope.imageSelected.classroomsToSend) {
        principalImage.classroomsToSend = $scope.imageSelected.classroomsToSend;
      }
      if (!!$scope.imageSelected.customFolders) {
        principalImage.customFolders = $scope.imageSelected.customFolders;
      }
      console.log("principalImage",principalImage);
      imagesInfo.$add(principalImage).then(function(ref){
        var promises = [];
        var refId = ref.key;
        if ($scope.imageSelected.folderType === 'custom') {
          for(var folderId in $scope.imageSelected.customFolders){
            if(!$scope.imageSelected.customFolders[folderId]) continue;
            promises.push(
              firebase.database().ref('galleryFolders/images').child($rootScope.allUserInfo.school).child(folderId).child(refId).set(returnObj)
            );
          }
        } else {
          for(var classroomId in $scope.imageSelected.classroomsToSend){
            if(!$scope.imageSelected.classroomsToSend[classroomId] || $rootScope.classSelected.id === classroomId) continue;
            promises.push(
              firebase.database().ref('gallery').child(classroomId).child(refId).set(returnObj)
            );
          }
        }
        $q.all(promises)
        .then(function () {
          $rootScope.hide();
          FirebasePluginFactory.logEvent('uploadPhotoVideo',{method:'one', type: 'photo'});
          $rootScope.showToast('imageUpdatedSuccess');
          $ionicHistory.goBack();
        });
      },function(e){
        console.log("failed add",e);
        alert($translate.instant('serverError'));
      });
    },function(err){
      if(err === 'maxSize'){
        alert($translate.instant('gallery_maxSize'));
      }else{
        alert($translate.instant('serverError'));
      }
      $rootScope.hide();
    });
  }

  function uploadGalleryVideo () {
    var fn, imagesInfo, idToUpload;
    if ($scope.imageSelected.folderType === 'custom') {
      fn = ImageFactory.uploadGalleryImageCustomFolder;
      idToUpload = Object.keys($scope.imageSelected.customFolders)[0];
      imagesInfo = ImageFactory.getCustomFolderImages(idToUpload, 1);
      delete $scope.imageSelected.classroomsToSend;
    } else {
      fn = ImageFactory.uploadGalleryImage;
      idToUpload = classroomId;
      imagesInfo = $rootScope.imagesInfo;
      delete $scope.imageSelected.customFolders;
    }
    fn($scope.imageSelected.image, idToUpload, mediaType).then(function(obj){
      $rootScope.hide();
      returnObj = obj;
      returnObj.push = true;
      if ($scope.imageSelected.message && $scope.imageSelected.message !== "") {
        returnObj.message = $scope.imageSelected.message;
      }
      console.log("returnObj",returnObj);
      imagesInfo.$add(returnObj).then(function(ref){
        var promises = [];
        var refId = ref.key;
        if (avoidCompress) {
          var avoidCompressObj = {
            filename: returnObj.filename,
            page: 'gallery',
            custom: ($scope.imageSelected.folderType === 'custom'),
            folder: idToUpload,
            school: $rootScope.allUserInfo.school,
            refId: refId
          };
          promises.push(
            firebase.database().ref('notificationCenter/videos').push(avoidCompressObj)
          );
        }
        if ($scope.imageSelected.folderType === 'custom') {
          for(var folderId in $scope.imageSelected.customFolders){
            if(!$scope.imageSelected.customFolders[folderId]) continue;
            promises.push(
              firebase.database().ref('galleryFolders/images').child($rootScope.allUserInfo.school).child(folderId).child(refId).set(returnObj)
            );
            if (avoidCompress) {
              var avoidCompressObj = {
                filename: returnObj.filename,
                page: 'gallery',
                custom: true,
                folder: folderId,
                school: $rootScope.allUserInfo.school,
                refId: refId
              };
              promises.push(
                firebase.database().ref('notificationCenter/videos').push(avoidCompressObj)
              );
            }
          }
        } else {
          for(var classroomId in $scope.imageSelected.classroomsToSend){
            if(!$scope.imageSelected.classroomsToSend[classroomId] || $rootScope.classSelected.id === classroomId) continue;
            promises.push(
              firebase.database().ref('gallery').child(classroomId).child(refId).set(returnObj)
            );
            if (avoidCompress) {
              var avoidCompressObj = {
                filename: returnObj.filename,
                page: 'gallery',
                custom: false,
                folder: classroomId,
                school: $rootScope.allUserInfo.school,
                refId: refId
              };
              promises.push(
                firebase.database().ref('notificationCenter/videos').push(avoidCompressObj)
              );
            }
          }
        }
        $q.all(promises)
        .then(function () {
          $rootScope.hide();
          FirebasePluginFactory.logEvent('uploadPhotoVideo',{method:'one', type: 'video'});
          $rootScope.showToast('imageUpdatedSuccess');
          $ionicHistory.goBack();
        });
      },function(e){
        console.log("failed add",e);
        alert($translate.instant('serverError'));
      });
    },function(err){
      if(err === 'maxSize'){
        alert($translate.instant('gallery_maxSize'));
      }else{
        alert($translate.instant('serverError'));
      }
      $rootScope.hide();
    });
  }

  $scope.sendImage = function () {
    if ($scope.imageSelected.folderType === 'custom') {
      for(var folderId in $scope.imageSelected.customFolders){
        if(!$scope.imageSelected.customFolders[folderId]) {
          delete $scope.imageSelected.customFolders[folderId];
        }
      }
      if (Object.keys($scope.imageSelected.customFolders).length === 0) {
        alert($translate.instant('gallery_selectFolders'));
        return;
      }
    } else {
      for(var classroomId in $scope.imageSelected.classroomsToSend){
        if(!$scope.imageSelected.classroomsToSend[classroomId]) {
          delete $scope.imageSelected.classroomsToSend[classroomId];
        }
      }
    }
    if ($scope.imageSelected.type === 'multiplePhotos') {
      startMultiplePhotos();
    } else {
      $rootScope.show(true);
      if (mediaType === 1) {
        pendingUploadVideo = true;
        if (hasToCompile) {
          try {
            if (isVideoCompressed) {
              uploadGalleryVideo();
            }
          } catch (error) {
            console.log('error', error);
            uploadGalleryVideo();
          }
        } else {
          uploadGalleryVideo();
        }
      } else {
        uploadGalleryImage();
      }
    }
  };

  $scope.$on("$ionicView.afterEnter", function(){
    $scope.imageSelected = $stateParams.imageSelected;
    console.log($scope.imageSelected);
    classroomId = $rootScope.classSelected.id;
    $scope.foldersCustom = ImageFactory.getCustomFolders($rootScope.allUserInfo.school);
    if ($scope.imageSelected.type === 'video') {
      mediaType = Camera.MediaType.VIDEO;
      isVideoCompressed = false;
      pendingUploadVideo = false;
      // if ($scope.imageSelected.imageUrl.toLowerCase().indexOf('.mp4') === -1 ) {
        // console.log('is not mp4');
        if (!!$rootScope.allUserInfo.avoidCompressVideo) {
          avoidCompress = true;
          console.log('is avoiding compress');
        } else {
          startUploadVideo();
          hasToCompile = true;
          avoidCompress = false;
        }
      // } else {
      //   hasToCompile = false;
      //   avoidCompress = false;
      // }
    } else {
      mediaType = Camera.MediaType.PICTURE;
    }
  });

  (function(){
  }());
})
