angular.module('childcareOnApp').controller('PhotosCtrl', function($scope, $rootScope, $translate, ImageFactory, $filter, $ionicActionSheet, ionicMaterialInk, ionicMaterialMotion, $state, $stateParams, configuration) {

  ionicMaterialInk.displayEffect();
  ionicMaterialMotion.ripple();
  var classroomId, loadedFolderId, imageUpladRef;
  $scope.folders = [];
  $scope.maxImages = 16;
  $scope.monthImages = [];

  $scope.deleteImageHelper = function(item){
    if($rootScope.allUserInfo.type === 'user') return;
    $translate(['gallery_deleteImageText','gallery_deleteImageTitle','login_forgotPassword_cancelButton','yes']).then(function(t){
      _confirm(t.gallery_deleteImageTitle,t.gallery_deleteImageText,t.login_forgotPassword_cancelButton,t.yes,
        function(buttonIndex){
        if(buttonIndex === 2) deleteImage(item);
      });
    });
  };

  var deleteImage = function(item){
    if(!$rootScope.imagesInfo && !$scope.images){
      alert($translate.instant('contentNotLoaded'));
      $rootScope.show();
      return;
    }
    if ($scope.type === 'custom') {
      var index = $scope.images.indexOf(item);
      $scope.images.$remove(item).then(function(){
        $rootScope.showToast('infoDeletedSuccessfully');
      },function(){
        alert($translate.instant('serverError'));
      });
    } else {
      var index = $rootScope.imagesInfo.indexOf(item);
      $rootScope.imagesInfo.$remove(index).then(function(){
        $rootScope.showToast('infoDeletedSuccessfully');
      }, function() {
        alert($translate.instant('serverError'));
      });
    }
  };

  $scope.selectPhoto = function(image){
    var custom;
    if ($scope.type === 'custom') {
      custom = {
        type: 'custom',
        folderId: $scope.customFolder.$id
      };
      var images = angular.copy($scope.images);
      var reversed = images.reverse();
      var index = 0;
      reversed.forEach(function(imageArray, i) {
        if (imageArray.location === image.location) {
          index = i;
        }
      });
      $state.go('app.multiImage',{img: reversed, index: index, canDelete: true, imageComplete: image, canDownload: (!!$rootScope.settings && !!$rootScope.settings.premium && !!$rootScope.settings.downloadGallery), custom: custom});
    } else {
      custom = {
        type: 'month'
      };
      var imagesInfo = angular.copy($rootScope.imagesInfo);
      imagesInfo = $filter('photos')(imagesInfo, $scope.monthDate).reverse();
      console.log('imagesInfo', imagesInfo);
      var index = 0;
      imagesInfo.forEach(function(imageArray, i) {
        if (imageArray.location === image.location) {
          index = i;
        }
      });
      $state.go('app.multiImage',{img: imagesInfo, index: index, canDelete: true, imageComplete: image, canDownload: (!!$rootScope.settings && !!$rootScope.settings.premium && !!$rootScope.settings.downloadGallery), custom: custom});
    }
  };

  $scope.selectVideo = function(image){
    var custom;
    if ($scope.type === 'custom') {
      custom = {
        type: 'custom',
        folderId: $scope.customFolder.$id
      };
    } else {
      custom = {
        type: 'month'
      };
    }
    var download = ($rootScope.settings && $rootScope.settings.downloadGallery);
    $state.go('app.videoViewer',{url: image.image, canDelete:true, videoComplete: image, canDownload: download, custom: custom});
  };

  $scope.downloadImage = function(imageToDownload){
    $rootScope.showToast('image_downloading');
    ImageFactory.downloadImgToGallery(imageToDownload.image, configuration.appName + " Img", imageToDownload.date);
  };

  $scope.loadMoreImages = function() {
    $rootScope.show();
    switch($scope.maxImages) {
      case 16: $scope.maxImages = 40; break;
      case 40: $scope.maxImages = 70; break;
      case 70: $scope.maxImages = 120; break;
      case 120: $scope.maxImages = 180; break;
      case 180: $scope.maxImages = 'full'; break;
    }
    $scope.images = ImageFactory.getCustomFolderImages($scope.customFolder.$id, $scope.maxImages);
    $scope.images.$loaded().then($rootScope.hide);
  };

  $scope.holdImage = function(image) {
    console.log('image holded!');
    var ionicActionSheet;
    var options = {
      buttons: [
        { text: '<i class="icon ion-eye"></i>'+ $translate.instant('diary_seeImage') },
        { text: '<i class="icon ion-android-download"></i>'+ $translate.instant('download_image') }
      ],
      titleText: $translate.instant('communications_images'),
      cancelText: $translate.instant('login_forgotPassword_cancelButton'),
      cancel: function() {
        // add cancel code..
      },
      buttonClicked: function(index) {
        console.log('button clicked', index);
        switch(index) {
          case 0: $scope.selectPhoto(image); break;
          case 1: $scope.downloadImage(image); break;
          default: console.log('default');
        }
        ionicActionSheet();
        return true;
      }
    };
    if ($rootScope.allUserInfo.type !== 'user') {
      options.destructiveText = '<i class="icon ion-trash-a"></i>'+ $translate.instant('gallery_deleteImageTitle');
      options.destructiveButtonClicked = function() {
        ionicActionSheet();
        $scope.deleteImageHelper(image);
      }
    }
    ionicActionSheet = $ionicActionSheet.show(options);
  };

  $scope.$on("$ionicView.afterEnter", function(){
    console.log($stateParams);
    $scope.type = (!$stateParams.monthDate && !!$stateParams.customFolder)? 'custom' : 'month';
    if ($scope.type === 'custom') {
      if (loadedFolderId !== $stateParams.customFolder.$id) {
        $scope.maxImages = 16;
        $scope.customFolder = $stateParams.customFolder;
        loadedFolderId = $scope.customFolder.$id;
        $scope.images = ImageFactory.getCustomFolderImages($scope.customFolder.$id, $scope.maxImages);
        console.log("customFolder",$scope.customFolder);
      }
    } else {
      $scope.monthDate = $stateParams.monthDate;
      console.log("monthDate",$scope.monthDate);
    }
  });

  (function(){
  }());
})
