angular.module('childcareOnApp').controller('FirstStepsPrincipalCtrl', function($scope, $rootScope, $translate, FeedbackFactory, ClassroomFactory, ChildFactory, CommonFactory, FirebasePluginFactory, $q, $ionicModal, $ionicHistory, $state, configuration) {

  $scope.mySchool = {};

  function createSchool() {
    return $q(function (resolve, reject) {
      FeedbackFactory.createFirstStepsSchool($scope.mySchool.name, $scope.mySchool.city, $scope.mySchool.phoneNumber, $scope.mySchool.comment)
      .then(function(id) {
        resolve(id);
      }, function(error) {
        alert($translate.instant('serverError'));
        reject();
      });
    });
  }
  function createClassroom(schoolId) {
    return $q(function(resolve, reject) {
      if (!$scope.mySchool.classroomName || $scope.mySchool.classroomName === '') {
        return resolve(null);
      }
      var classroom = {
        name: $scope.mySchool.classroomName,
        type: {
          value: 1
        },
        options: {
          bottles: true
        }
      };
      ClassroomFactory.addClassroomFirstSteps(classroom, schoolId)
      .then(function(classroomId) {
        resolve(classroomId);
      });
    });
  }
  function createChild(schoolId, classroomId) {
    return $q(function(resolve, reject) {
      if (
        !$scope.mySchool.classroomName || $scope.mySchool.classroomName === '' ||
        !$scope.mySchool.childName || $scope.mySchool.childName === ''        
      ) {
        return resolve(null);
      }
      var child = {
        name: $scope.mySchool.childName,
        surname: '',
        birthdate: '10/10/2010',
        birthdateParsed: '10_10',
        gender: 'female',
        classroomId: classroomId,
        classroomName: $scope.mySchool.classroomName,
        school: schoolId
      };
      ChildFactory.createFirstStepsChild(child)
      .then(function() {
        resolve();
      });
    });
  }
  function sendServerRequest(schoolId) {
    return $q(function (resolve, reject) {
      var obj = {
        schoolId: schoolId,
        id: $rootScope.allUserInfo.id
      };
      if (!!$scope.mySchool.comment && $scope.mySchool.comment !== '') {
        obj.comment = $scope.mySchool.comment;
      }
      var url = configuration.url.schoolCreated;
      console.log('configuration.schoolCreated', configuration.schoolCreated);
      console.log('configuration', configuration);
      console.log('send server request', obj, url);
      CommonFactory.serverRequest(url, obj);
      return resolve();
    });
  }
  function loadData() {
    return $q(function (resolve, reject) {
      $rootScope.getTeacherInfo()
      .then(function() {
        resolve();
      });
    });
  }

  $scope.start = function() {
    $scope.closeModal();
    $ionicHistory.nextViewOptions({
      disableBack: true,
      historyRoot: true
    });
    $state.go('app.home', {location:'replace'});
  };

  $scope.createTestSchool = function(){
    console.log("$scope.mySchool",$scope.mySchool);
    if(!$scope.mySchool.name || !$scope.mySchool.city || !$scope.mySchool.phoneNumber){
      alert($translate.instant('fillFields'));
      return;
    }
    $scope.schoolCreating = 0;
    $scope.openModal();
    createSchool()
    .then(function(schoolId) {
      $scope.schoolCreating = 40;
      createClassroom(schoolId)
      .then(function(classroomId) {
        $scope.schoolCreating = 70;
        createChild(schoolId, classroomId)
        .then(function() {
          $scope.schoolCreating = 90;
          $rootScope.getTeacherInfo()
          .then(function() {
            $scope.schoolCreating = 100;
            // $scope.closeModal();
            FeedbackFactory.saveSchoolLog(schoolId)
            .then(function() {
              $scope.mySchool = {};
              FirebasePluginFactory.setUserProperty('type','principal');
            });
          });
        });
      })
    });
  };

  $ionicModal.fromTemplateUrl('modal-creating.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });
  $scope.openModal = function() {
    $scope.modal.show();
  };
  $scope.closeModal = function() {
    $scope.modal.hide();
  };
  // Cleanup the modal when we're done with it!
  $scope.$on('$destroy', function() {
    if (!!$scope.modal) {
      $scope.modal.remove();
    }
  });

  (function(){
  }());
});
