angular.module('childcareOnApp').controller('PaymentsCtrl', ['$scope', '$rootScope', '$translate', '$ionicActionSheet', '$state', '$ionicModal', '$q', 'PaymentsFactory', 'ImageFactory', PaymentsCtrl]);

function PaymentsCtrl($scope, $rootScope, $translate, $ionicActionSheet, $state, $ionicModal, $q, PaymentsFactory, ImageFactory) {

  var vm = this;
  var childId;
  vm.docDisabled = false;
  vm.descriptionOpened = false;
  vm.fileUploaded = false;
  vm.docRoute = "";
  vm.progress = {};
  vm.uploading = {};
  vm.childrenToSend = {};
  vm.addPaymentParams = {};
  var inAppBrowserRef;

  function uploadDocument (file){
    return $q(function(resolve,reject){
      if(typeof file === 'undefined'){
        reject('undefined');
        return;
      }
      if(file.indexOf('application/pdf') === -1){
        alert($translate.instant('communications_noPdf'));
        reject('noPdf');
      }
      file = file.split('base64,');
      file = file[file.length -1];
      $scope.$evalAsync(function () {
        vm.uploading.document = true;
        vm.progress.document = 0;
      });

      var interval = setInterval(function() {
        $scope.$evalAsync(function () {
          vm.progress.document = ImageFactory.getPercentageProgress();
        });
      }, 200);

      ImageFactory.uploadDocumentDefault(file, "application/pdf", 'payments', false).then(function(url){
        docLink = url;
        vm.addPaymentParams.document = url;
        console.log('file uploaded', vm.addPaymentParams);
        delete $rootScope.documentUrl;
        clearInterval(interval);
        resolve();
      },function(){
        clearInterval(interval);
        reject('error');
      });
    });
  }

  function onSuccess(imageData) {
    console.log("success",imageData);
    var date = new Date().getTime();
    vm.uploading.image = true;
    vm.fileUploaded = true;
    var interval = setInterval(function() {
      $scope.$evalAsync(function () {
        vm.progress.image = ImageFactory.getPercentageProgress();
      });
    }, 200);
    ImageFactory.saveImage(imageData, 'payments/' + $rootScope.allUserInfo.school, date, 'image/jpeg', false).then(function(url){
      console.log("imagen subida",url);
      $rootScope.showToast("imageUpdatedSuccess");
      if(!vm.addPaymentParams.images) {
        vm.addPaymentParams.images = {};
      }
      vm.addPaymentParams.images[date] = {image: url};
      vm.uploading.image = false;
      delete vm.progress.image;
      clearInterval(interval);
      setTimeout(function() {
        $scope.$evalAsync(function() {
          vm.fileUploaded = false;
        });
      }, 3000);
    },function(){
      clearInterval(interval);
      alert($translate.instant('serverError'));
      vm.uploading.image = false;
    });
  }

  vm.uploadImage = function(){
    if (!vm.addPaymentParams.images) {
      vm.addPaymentParams.images = {};
    }
    if(Object.keys(vm.addPaymentParams.images).length >= 4){
      return alert($translate.instant('auth_maximumImages'));
    }
    $translate(['communications_selectImage','communications_imageTitle','communications_camera','communications_gallery','login_forgotPassword_cancelButton']).then(function(t){
      navigator.notification.confirm(
        t.communications_selectImage, // message
         function(index){
           if(index === 2) return;
           var source = (index === 1)? Camera.PictureSourceType.CAMERA: Camera.PictureSourceType.PHOTOLIBRARY;
           navigator.camera.getPicture(onSuccess, null, { correctOrientation:true,quality: 80,targetWidth: 1080, targetHeight:1020, sourceType: source,
             destinationType: Camera.DestinationType.DATA_URL, encodingType: Camera.EncodingType.JPEG
           });
         },
        t.communications_imageTitle,           // title
        [t.communications_camera, t.login_forgotPassword_cancelButton, t.communications_gallery]     // buttonLabels
      );
    });
  };

  vm.openDescription = function(id) {
    if (vm.descriptionOpened === id) {
      vm.descriptionOpened = false;
    } else {
      $scope.$evalAsync(function() {
        vm.descriptionOpened = id;
      });
    }
  };

  vm.finishUpload = function(file) {
    if (typeof file === 'undefined' || !file) {
      return false;
    }
    console.log("finish upload");
    uploadDocument(file).then(function(){
      $scope.$evalAsync(function() {
        vm.docDisabled = true;
        vm.uploading.document = false;
      });
    },function(error){
      if(error === 'error') alert($translate.instant('serverError'));
    });
  };

  vm.sendAddPayment = function(){
    if(!vm.addPaymentParams.title || vm.addPaymentParams.title === '' || !vm.addPaymentParams.deadlineUnparsed || vm.addPaymentParams.deadlineUnparsed === ''){
      alert($translate.instant('fillFields'));
      return;
    }
    var childrenToSend = [];
    for(var i in vm.childrenToSend){
      if(vm.childrenToSend[i] === true){
        childrenToSend.push(i);
      }
    }
    if (childrenToSend.length === 0) {
      alert($translate.instant('fillFields'));
      return;
    }
    vm.addPaymentParams.date = new Date().getTime();
    vm.addPaymentParams.deadline = vm.addPaymentParams.deadlineUnparsed.getTime();
    var notificationToSend = {
      deadline: vm.addPaymentParams.deadline,
      ids: childrenToSend,
      title: vm.addPaymentParams.title
    };
    PaymentsFactory.addNotification(notificationToSend)
    .then(function (notificationReferences) {
      var notificationIds = notificationReferences.map(function (notificationRef) {
        return notificationRef.key;
      });
      PaymentsFactory.add(vm.addPaymentParams, childrenToSend, notificationIds)
      .then(function () {
        $rootScope.showToast('infoSavedSuccessfully');
        vm.closeModal();
      }, function() {
        $rootScope.showToast('serverError');
      });
    });
  };

  vm.openLink = function(item) {
    var url = item.link;
    if (url.indexOf('http') === -1) {
      url = 'https://' + url;
    }
    window.open(url, '_system', 'location=yes');
  };

  $ionicModal.fromTemplateUrl('modal-add.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    vm.modalAdd = modal;
  });

  $scope.$on('$destroy', function() {
    if (!!vm.modalAdd) {
      vm.modalAdd.remove();
    }
  });

  vm.closeModal = function() {
    vm.addPaymentParams = {};
    vm.childrenToSend = {};
    vm.docDisabled = false;
    vm.modalAdd.hide();
  };

  vm.seeImage = function(image) {
    $state.go('app.imageViewer',{img:image.image, canDelete: false, canDownload:true, imageComplete: image});
  };

  vm.seeDocument = function(document) {
    var doc = {
      name: document.date.toString(),
      url: document.document
    };
    $rootScope.show();
    ImageFactory.openDocument(doc)
    .then(function() {
      $rootScope.hide();
    }, function() {
      $rootScope.hide();
      window.open(document.document, '_system', 'location=yes');
    });
  };

  vm.deletePayment = function(item) {
    _confirm($translate.instant('button_delete'), $translate.instant('confirmRemoveGeneric') ,$translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_delete'), function(buttonIndex){
      if(buttonIndex === 2){
        $q.all([
          PaymentsFactory.removeNotification(item.notificationId),
          vm.payments.$remove(item)
        ])
        .then(function () {
          $rootScope.showToast('infoDeletedSuccessfully');
        }, function () {
          alert($translate.instant('serverError'));
        });
      }
    });
  };

  vm.openModal = function() {
    var deadline = new Date();
    deadline.setMonth(deadline.getMonth() +1);
    vm.addPaymentParams.deadlineUnparsed = deadline;
    vm.modalAdd.show();
  };

  vm.optionsPayment = function(item) {
    var ionicActionSheet;
    var options = {
      buttons: [
        // { text: '<i class="icon ion-eye"></i>'+ $translate.instant('auth_seeAnswers') },
        // { text: '<i class="icon ion-android-download"></i>'+ $translate.instant('catalog_description') }
      ],
      titleText: item.title,
      cancelText: $translate.instant('login_forgotPassword_cancelButton'),
      cancel: function() {
        // add cancel code..
      },
      buttonClicked: function(index) {
        console.log('button clicked', index);
        switch(index) {
          case 0: vm.openModalAnswers(item); break;
          case 1: vm.openDescription(item.$id); break;
          default: console.log('default');
        }
        ionicActionSheet();
        return true;
      }
    };
    if ($rootScope.allUserInfo.type === 'principal') {
      options.destructiveText = '<i class="icon ion-trash-a"></i>'+ $translate.instant('button_delete');
      options.destructiveButtonClicked = function() {
        vm.deletePayment(item);
        ionicActionSheet();
      }
    }
    ionicActionSheet = $ionicActionSheet.show(options);
  };

  $scope.$on("$ionicView.afterEnter", function(){
    console.log("$rootScope.students",$rootScope.students);
    childId = ($rootScope.allUserInfo.type === 'user')? $rootScope.childSelected : $rootScope.studentSelected.id;
    if(!vm.payments || vm.payments.$id !== childId){
      vm.payments = PaymentsFactory.get(childId);
    }
  });

  (function(){
  }());
}