angular.module('childcareOnApp').directive('autoComplete', function($timeout) {
  return function(scope, iElement, iAttrs) {
    iElement.autocomplete({
      source: scope[iAttrs.uiItems],
      select: function() {
        $timeout(function() {
          iElement.trigger('input');
        }, 0);
      }
    });
  };
})

.directive('fallbackSrc', function ($rootScope, CommonFactory) {
  var fallbackSrc = {
    link: function postLink(scope, iElement, iAttrs) {
      iElement.bind('error', function() {
        if(iAttrs.intents > 0) return;
        angular.element(this).attr("src", iAttrs.fallbackSrc);
        console.log("intents",iAttrs.intents);
        iAttrs.intents ++;
        // if(iAttrs.typeImage === 'profileImage')
        // $log.error('failed using image');
      });
    }
   }
   return fallbackSrc;
})

.directive("fileread", [ '$rootScope','$translate', 'CommonFactory', function ($rootScope, $translate, CommonFactory) {
  return {
    restrict: 'A',
      scope: {
          fileread: '=',
          onFinish: '='
      },
      bindToController: true,
      link: function (scope, element, attributes) {
          element.bind("change", function (changeEvent) {
              var reader = new FileReader();
              reader.onload = function (loadEvent) {
                if(parseInt(loadEvent.loaded) > 12000000){
                  alert($translate.instant ('communications_sizeExceeded'));
                  return;
                }
                scope.$apply(function () {
                  scope.fileread = loadEvent.target.result;
                  $rootScope.documentUrl = loadEvent.target.result;
                });
                scope.filereadController.fileread = loadEvent.target.result;
                if (typeof scope.filereadController.onFinish === 'function') {
                  scope.filereadController.onFinish(loadEvent.target.result);
                }
              }
              reader.error = function (e) {
                CommonFactory.saveLog({
                  user: $rootScope.allUserInfo.id,
                  error: "Error in reader",
                  errorCode:e.target.error.code,
                  errorMessage:e.target.error
                });
                var errorText;
                switch(e.target.error.code) {
                  case e.target.error.NOT_FOUND_ERR:
                    errorText = 'NOT_FOUND_ERR'
                    break;
                  case e.target.error.NOT_READABLE_ERR:
                  errorText = 'NOT_READABLE_ERR';
                    break;
                  case e.target.error.ABORT_ERR:
                  errorText = 'ABORT_ERR';
                    break; 
                  case e.target.error.SECURITY_ERR:
                  errorText = 'SECURITY_ERR';
                    break;
                  case e.target.error.ENCODING_ERR:
                  errorText = 'ENCODING_ERR';
                    break;

                  default:
                  errorText = 'DEFAULT';

                }
                alert($translate.instant('uploadDocuments_error_previewText') + ": " + $translate.instant('uploadDocuments_error_'+errorText));
              }
              reader.readAsDataURL(changeEvent.target.files[0]);
          });
      },
      controller: ['$scope', function($scope) {
      }],
      controllerAs: 'filereadController'
  }
}])

.directive("newFileread", [ '$rootScope','$translate', 'CommonFactory', function ($rootScope, $translate, CommonFactory) {
  return {
    restrict: 'A',
      scope: {
          fileread: '=',
          onFinish: '='
      },
      bindToController: true,
      link: function (scope, element, attributes) {
          element.bind("change", function (changeEvent) {
              if (typeof scope.filereadController.onFinish === 'function') {
                scope.filereadController.onFinish(changeEvent.target.files[0]);
              }
          });
      },
      controller: ['$scope', function($scope) {
      }],
      controllerAs: 'filereadController'
  }
}])

.directive('searchBar',function(){
  return {
    restrict: 'E',
    scope: {
      search:'=',
    },
    templateUrl: 'app/js/directives/search/search.html',
    controller: ['$scope', function($scope) {

    }]
  };
})

.filter('toArray', function() {
  return function(obj) {
    var result = [];
    angular.forEach(obj, function(val) {
      result.push(val);
    });
   return result;
 }
})

.directive('timeToDate',['ChildcareOnUtils', function (ChildcareOnUtils) {
  return {
      require: 'ngModel',
      link: function(scope, ele, attr, ngModel) {
          // converts DOM value to ng-model
          ngModel.$parsers.push(function(value) {
            // console.log("value is",value);
              // var date = Date.parse(value);
              try {
                var time = ChildcareOnUtils.addLeadingZeros(value.getHours()) + ":" + ChildcareOnUtils.addLeadingZeros(value.getMinutes());
                return time;
              } catch (error) {
                return '00:00';
              }
          });

          // converts ng-model to DOM value
          ngModel.$formatters.push(function(value) {
              // var date = new Date(value);
              // return date;
              if(typeof value === 'undefined' || value.length !== 5) return "";
              var hours = value.substr(0,2);
              var minutes = value.substr(3,2);
              var date = new Date();
              date.setHours(parseInt(hours));
              date.setMinutes(parseInt(minutes));
              date.setSeconds(0);
              date.setMilliseconds(0);
              return date;
          });
      }
  }
}])

.directive('timestampToDate', function() {
  return {
      require: 'ngModel',
      link: function(scope, ele, attr, ngModel) {
          // converts DOM value to ng-model
          ngModel.$parsers.push(function(value) {
              return Date.parse(value);
          });

          // converts ng-model to DOM value
          ngModel.$formatters.push(function(value) {
              var date = new Date(value);
              return date;
          });
      }
  }
})

.directive('birthdayCard', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/home/cards/birthday-card.html',
    controller: ['$scope','$rootScope', '$translate', 'ChildFactory', function($scope, $rootScope, $translate, ChildFactory) {
      ChildFactory.getBirthdays()
      .then(function(birthdays) {
        $scope.birthday = birthdays;
      }, function(birthdays) {
        $scope.birthday = birthdays;
      });
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  };
})

.directive('noChildren', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/home/cards/no-children.html',
    controller: ['$scope','$rootScope', '$translate', '$sce', function($scope, $rootScope, $translate, $sce) {
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  };
})

.directive('medals', function() {
  return {
    restrict: 'E', //attribute or element
    transclude: true,
    scope: {
      medals:'=',
      modal:'=',
      selected:'='
    },
    templateUrl: 'app/components/diary/medals.html',
    controller: ['$scope','$rootScope', '$translate', function($scope, $rootScope, $translate) {

      $scope.comment = {};
      $scope.medals.$loaded().then(function () {
        var medalsFilled = {};
        $scope.medals.forEach(function (element, i) {
          medalsFilled[i] = true;
          $scope.comment[i] = element.comment;
        });
        $scope.medalsFilled = medalsFilled;
      });

      $scope.selectMedal = function (typeOfMedal) {
        if ($rootScope.allUserInfo.type === 'user') {
          if (!$scope.medals.hasOwnProperty(typeOfMedal)) return $rootScope.showToast('diary_progressNotAchieved');
          $scope.selected.medal = typeOfMedal;
          $scope.selected.icon = (typeOfMedal === 'cup')? 'trophy':'medal';
          console.log("selected",$scope.selected);
          $scope.modal.show();
        } else {
          if ($scope.medalSelected !== typeOfMedal) $scope.medalSelected = typeOfMedal;
          else $scope.medalSelected = false;
        }
      };
      $scope.sendMedal = function () {
        if (!$scope.comment[$scope.medalSelected]) return alert($translate.instant('fillFields'));
        _confirm($translate.instant('meeting_confirm'), $translate.instant('diary_confirmProgress'), $translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_accept'), function(buttonIndex){
          if(buttonIndex === 2){
            $scope.medals[$scope.medalSelected] = {
              comment: $scope.comment[$scope.medalSelected],
              date: new Date().getTime(),
              senderId: $rootScope.allUserInfo.id
            };
            $scope.medals.$save()
            .then(function () {
              $scope.medalSelected = false;
              $scope.medalsFilled[$scope.medalSelected] = true;
              alert('🏅 ' + $translate.instant('diary_progressSaved') + ' 🏅');
            }, function (error) {
              console.log("error save medals",error);
              delete $scope.medals[$scope.medalSelected];
              alert($translate.instant('serverError'));
            }); 
          }
        });
      };

    }],
    link: function($scope, elem, attr, ctrl) {
    }
  };
})

.directive('teacherAvailability', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/home/cards/teacher-availability.html',
    controller: ['$scope','$rootScope', '$translate', 'NotificationFactory', function($scope, $rootScope, $translate, NotificationFactory) {
      NotificationFactory.childTeachersAvailable.set($rootScope.allUserChilds[$rootScope.childSelected]);
      $scope.childTeachersAvailable = NotificationFactory.childTeachersAvailable.get;
    }],
    link: function($scope, elem, attr, ctrl) {
    }
  };
})

.directive('deactivedApp', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/home/cards/deactived-app.html',
    controller: ['$scope', 'configuration', function($scope, configuration) {
      $scope.appName = configuration.appName;
    }],
    link: function($scope, elem, attr, ctrl) {
    }
  };
})

.directive('inputResize', function() {
  return {
    restrict: 'A',
    link: function($scope, element, attr, ctrl) {
      element.on('keyup', function() {
        if (!attr.limitHeight || element[0].scrollHeight < attr.limitHeight) {
          element.css('overflow', 'hidden');
          element.css('height', 'auto');
          element.css('height', element[0].scrollHeight + 'px');
        }
      });
    }
  };
})

.directive('pollModal', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/poll/poll-modal.html',
    controller: ['$scope','$rootScope', 'FeedbackFactory', '$state', function($scope, $rootScope, FeedbackFactory, $state) {
      $scope.show = false;

      function getPoll() {
        FeedbackFactory.getPoll()
        .then(function(result) {
          $scope.show = !result;
        }, function() {
          $scope.show = false;
        });
      }

      $scope.hide = function() {
        $scope.show = false;
      };

      $scope.goToPoll = function() {
        $scope.show = false;
        $state.go('app.poll');
      };

      (function() {
        console.log('$rootScope', $rootScope.allUserInfo.id);

        FeedbackFactory.getIfPollActivated()
        .then(function (pollValue) {
          console.log('pollValue', pollValue);
          if (pollValue) {
            getPoll();
          }
        }, function () {
          console.log('not loading poll');
        });
      })();
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  };
})

.directive('suggestions',function(){
  return {
    restrict: 'E',
    scope: {
      suggestionsList: '=',
      clickElement: '='
    },
    templateUrl: 'app/components/diary/suggestions.html',
    controller: ['$scope', '$translate', function($scope, $translate) {
      $scope.setSuggestion = function(item) {
        if (typeof $scope.clickElement === 'string' && $scope.clickElement.length > 0) {
          _confirm($translate.instant('suggestions_confirm'), $translate.instant('confirmGeneric') ,$translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_accept'), function(buttonIndex){
            if(buttonIndex === 2){
              $scope.$evalAsync(function() {
                $scope.clickElement = item;
              });
            }
          });
        } else {
          $scope.clickElement = item;
        }
      };
    }]
  };
})

.directive('datetimeToTime', function () {
  return {
      require: 'ngModel',
      link: function(scope, ele, attr, ngModel) {
          // converts DOM value to ng-model
          ngModel.$parsers.push(function(value) {
            console.log("value is",value);
            // var date = Date.parse(value);
            // return ChildcareOnUtils.addLeadingZeros(value.getHours()) + ":" + ChildcareOnUtils.addLeadingZeros(value.getMinutes());
            return value.getTime();
          });

          // converts ng-model to DOM value
          ngModel.$formatters.push(function(value) {
              // var date = new Date(value);
              // return date;
              return new Date(value);
          });
      }
  }
})

.directive('documents',function(){
  return {
    restrict: 'E',
    scope: {
      documentList: '=',
      modal: '=',
      title: '=',
    },
    templateUrl: 'app/js/directives/documents/documents.html',
    controller: ['$scope', '$translate', '$rootScope', '$ionicActionSheet', '$firebaseStorage', '$q', 'ImageFactory', '$http', function($scope, $translate, $rootScope, $ionicActionSheet, $firebaseStorage, $q, ImageFactory, $http) {
      console.log('received documentList', $scope.documentList);
      console.log('received modal', $scope.modal);
      console.log('received title', $scope.title);
      console.log('received test', $scope.test);
      $scope.isEdit = false;
      $scope.removeDocument = function(item) {
        _confirm($translate.instant('button_delete'), $translate.instant('confirmRemoveGeneric') ,$translate.instant('login_forgotPassword_cancelButton'), $translate.instant('button_delete'), function(buttonIndex){
          if(buttonIndex === 2){
            $scope.documentList.$remove(item)
            .then(function () {
              $rootScope.showToast('infoDeletedSuccessfully');
            }, function () {
              alert($translate.instant('serverError'));
            });
          }
        });
      };
      $scope.openDocument = function(document){
        console.log("item is",document);
        if (typeof window.cordova === 'undefined') {
          window.open(document.url, '_system', 'location=true');
        }
        if (document.url.indexOf('http') === -1) {
          document.url = 'https://' + document.url;
        }
        $rootScope.show();
        ImageFactory.openDocument(document)
        .then(function() {
          $rootScope.hide();
        }, function() {
          $rootScope.hide();
          window.open(document.url, '_system', 'location=true');
        });
      };

      $scope.clickDelete = function() {
        $scope.$evalAsync(function() {
          $scope.isEdit = !$scope.isEdit;
        });
      };

      $scope.clickMore = function(item) {
        var ionicActionSheet;
        var options = {
          buttons: [
            { text: '<i class="icon ion-eye"></i>'+ $translate.instant('button_see') },
            { text: '<i class="icon ion-android-download"></i>'+ $translate.instant('button_download') }
          ],
          titleText: item.title,
          cancelText: $translate.instant('login_forgotPassword_cancelButton'),
          cancel: function() {
            // add cancel code..
          },
          buttonClicked: function(index) {
            console.log('button clicked', index);
            switch(index) {
              case 0: $scope.openDocument(item); break;
              case 1: window.open(item.url, '_system', 'location=false'); break;
              default: console.log('default');
            }
            ionicActionSheet();
            return true;
          }
        };
        if ($rootScope.allUserInfo.type === 'principal') {
          options.destructiveText = '<i class="icon ion-trash-a"></i>'+ $translate.instant('documents_delete');
          options.destructiveButtonClicked = function() {
            $scope.removeDocument(item);
            ionicActionSheet();
          }
        }
        ionicActionSheet = $ionicActionSheet.show(options);
      };
    }]
  };
})

.directive('menusScheduleCard', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
    },
    templateUrl: 'app/components/home/cards/menus-schedule-card.html',
    controller: ['$scope', '$rootScope', 'ChildcareOnUtils', 'SchedulesFactory', 'MenusFactory', function($scope, $rootScope, ChildcareOnUtils, SchedulesFactory, MenusFactory) {

      $scope.todayDateParsed = ChildcareOnUtils.parseDateToNumbers(new Date());
      $scope.todayDay = ChildcareOnUtils.getTodayDate(new Date());
      $scope.mealOrder = ['first','second','third','fourth','fifth'];
      console.log('$rootScope.settings', $rootScope.settings);
      $scope.getIfShow = function(page){
        if(!$rootScope.settings) return true;
        switch(page){
          case "schedule":
          case "menus": return !($rootScope.settings[page]);
          break;
          case "all": return !($rootScope.settings.schedule && $rootScope.settings.menus);
          break;
          default: return true;
        }
      };

      var classroomId = $rootScope.allUserChilds[$rootScope.childSelected].classroomId;
      SchedulesFactory.getSchedule(classroomId).then(function(schedule){
        console.log("schedule success",schedule);
        $scope.schedule = schedule;
      },function(){
        console.log("error en getSchedule");
      });
      MenusFactory.getUserMenus(classroomId).then(function(menus){
        console.log("getMenus success",menus);
        $scope.menus = menus;
      },function(){
        console.log("error en getMenus");
      });
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  };
})


.directive('galleryButtons', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
      type: '='
    },
    templateUrl: 'app/js/directives/gallery-buttons/gallery-buttons.html',
    controller: ['$scope', '$state', '$rootScope', '$q', '$filter', '$translate', function($scope, $state, $rootScope, $q, $filter, $translate) {
      console.log('$scope.type', $scope.type);

      var imageSelected, thumbnails;

      function lookIfVideoAlreadyUploaded(maximumNumberOfVideos){
        if (!!$rootScope.settings.gallery && !!$rootScope.settings.gallery.premium) {
          return false;
        }
        var maximumPhotos = $filter('maximumPhotos')($rootScope.imagesInfo);
        var videos = maximumPhotos.filter(function(item) {
          return (item.type === 'video' && !item.hide);
        });
        return (videos.length >= maximumNumberOfVideos);
      }

      function lookIfImageAlreadyUploaded() {
        if ($rootScope.settings.premium || (!!$rootScope.settings.gallery && !!$rootScope.settings.gallery.premium)) {
          return false;
        }
        var maximumPhotos = $filter('maximumPhotos')($rootScope.imagesInfo);
        var fotos = maximumPhotos.filter(function(item) {
          return (item.type !== 'video' && !item.hide);
        });
        return (fotos.length >= 50);
      }

      function getCompressedImages(medias) {
        return $q(function (resolve, reject) {
          thumbnails = [];
          var promises = [];
          for (var i = 0; i < medias.length; i++) {
            promises.push(compressImage(medias[i]));
          }
          $q.all(promises)
          .then(function (result) {
            resolve();
          });
        });
      }
    
      function compressImage(media) {
        return $q(function (resolve, reject) {
          media.quality = 50;
          MediaPicker.compressImage(media, function(imageCompressed) {
            console.log('imageCompressed', imageCompressed);
            MediaPicker.fileToBlob(imageCompressed.path, function(data) {
              thumbnails.push({
                path: imageCompressed.path,
                file: data,
                index: imageCompressed.index
              });
              resolve();
            });
          }, function(e) {
            console.log(e);
            return resolve();
          });
        });
      }

      function onSuccess(imageData) {
        console.log("success",imageData);
        if (mediaType === 1) {
          imageSelected.image = imageData;
          imageSelected.imageUrl = 'file://' + imageData;
          imageSelected.type = 'video';
        } else {
          imageSelected.image = imageData;
          imageSelected.imageUrl = "data:image/jpeg;base64," + imageData;
          imageSelected.type = 'image';
        }
        var classroomsToSend = {};
        classroomsToSend[$rootScope.classSelected.id] = true;
        imageSelected.classroomsToSend = classroomsToSend;
        $state.go('app.newImage', {imageSelected: imageSelected});
      }

      function onFail(message) {
        console.log('Failed because: ' + message);
        if(message.indexOf('error') !== -1) alert($translate.instant('imageUpdatedError'));
      }

      $scope.uploadPhoto = function(type, recordType){
        imageSelected = {
          customFolders: {},
          folderType: $scope.type
        };
        if(!$rootScope.imagesInfo || !$rootScope.imagesInfo.hasOwnProperty('$ref')){
          alert($translate.instant('contentNotLoaded'));
          $rootScope.show();
          return;
        }
        var maximumNumberOfVideos = ($rootScope.settings.premium) ? 10 : 1;
        if(recordType === 'video' && lookIfVideoAlreadyUploaded(maximumNumberOfVideos)) return alert($translate.instant('gallery_videoVariableAlreadyUpload', { number: maximumNumberOfVideos }));
        if(recordType !== 'video' && lookIfImageAlreadyUploaded()) return alert($translate.instant('gallery_imageAlreadyUpload'));
        var source = Camera.PictureSourceType.CAMERA;
        if(type === 'gallery') source = Camera.PictureSourceType.PHOTOLIBRARY;
        mediaType = Camera.MediaType.PICTURE;
        if(recordType === 'video') {
          mediaType = Camera.MediaType.VIDEO;
        }
        console.log("mediaType",mediaType);
    
        navigator.camera.getPicture(onSuccess, onFail, { correctOrientation:true, quality: 60,targetWidth: 1024, targetHeight: 1024, sourceType: source,
          destinationType: Camera.DestinationType.DATA_URL, encodingType: Camera.EncodingType.JPEG, mediaType: mediaType
        });
      };

      $scope.uploadMultiplePhotos = function(){
        var args = {
          'selectMode': 100, //101=picker image and video , 100=image , 102=video
          'maxSelectCount': 10, //default 40 (Optional)
          'maxSelectSize': 188743680, //188743680=180M (Optional)
        };
        mediaType = Camera.MediaType.PICTURE;
        MediaPicker.getMedias(args, function(medias) {
          //medias [{mediaType: "image", path:'/storage/emulated/0/DCIM/Camera/20170808_145202.jpg', size: 21993}]
          if (!medias || medias.length === 0) {
            return console.log('no images selected');
          }
          getCompressedImages(medias)
          .then(function() {
            var classroomsToSend = {};
            classroomsToSend[$rootScope.classSelected.id] = true;
            imageSelected = {
              image: thumbnails,
              type: 'multiplePhotos',
              numberPhotos: medias.length,
              customFolders: {},
              folderType: $scope.type,
              classroomsToSend: classroomsToSend
            };
            $state.go('app.newImage', {imageSelected: imageSelected});
          });
        }, function(e) { console.log(e) });
      };
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  };
})

.directive('childPhoto', function() {
  return {
    restrict: 'E', //attribute or element
    scope: {
      studentId: '='
    },
    templateUrl: 'app/js/directives/child-photo/child-photo.html',
    controller: ['$scope', '$localStorage', 'ChildcareOnUtils', '$firebaseObject', function($scope, $localStorage, ChildcareOnUtils, $firebaseObject) {
      var studentLocal = $localStorage.getCHObject('photo_' + $scope.studentId);
      var photo;
      if (!!studentLocal.time && ChildcareOnUtils.isSameDay(new Date(), new Date(parseInt(studentLocal.time)))) {
        if (studentLocal.obj.photo) {
          photo = {
            $value: studentLocal.obj.photo
          };
        } else {
          photo = {
            $value: false
          };
        }
      } else {
        photo = $firebaseObject(firebase.database().ref('child').child($scope.studentId).child('photo'));
        photo.$loaded()
        .then(function() {
          var value = false;
          if (!!photo.$value) {
            value = photo.$value;
          }
          var obj = {
            photo: photo.$value
          };
          $localStorage.setCHObject('photo_' + $scope.studentId, obj, $scope.studentId);
        });
      }
      $scope.photo = photo;
    }],
    link: function($scope, elem, attr, ctrl) {
      
    }
  }
});
