angular.module('childcareOnApp').factory('ImageFactory', function($rootScope, $q, $firebaseArray, $firebaseObject, $firebaseStorage, CommonFactory, $translate, configuration){

  var uploadPercentage = 0;
  var loadedSchoolId;

  var convertUrlToBase64 = function(url) {
    return $q(function (resolve, reject) {
      var xhr = new XMLHttpRequest();
      xhr.onload = function() {
        var reader = new FileReader();
        reader.onloadend = function() {
          resolve(reader.result);
        }
        reader.readAsDataURL(xhr.response);
      };
      xhr.open('GET', url);
      xhr.responseType = 'blob';
      xhr.send();
    });
  };

  var b64toBlob = function(b64Data, contentType, sliceSize){
    contentType = contentType || '';
    sliceSize = sliceSize || 512;

    var byteCharacters = atob(b64Data);
    var byteArrays = [];

    for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
      var slice = byteCharacters.slice(offset, offset + sliceSize);

      var byteNumbers = new Array(slice.length);
      for (var i = 0; i < slice.length; i++) {
        byteNumbers[i] = slice.charCodeAt(i);
      }

      var byteArray = new Uint8Array(byteNumbers);

      byteArrays.push(byteArray);
    }

    var blob = new Blob(byteArrays, {type: contentType});
    return blob;
  };

  var getDate = function(){
    var date = new Date().getTime().toString();
    return date.slice(0,date.length-3);
  };

  function uploadProcess(uploadTask, shouldShowToast){
    return $q(function(resolve,reject){
      if(typeof shouldShowToast === 'undefined') shouldShowToast = true;
      uploadTask.$progress(function(snapshot) {
        var percentUploaded = (snapshot.bytesTransferred / snapshot.totalBytes) * 100;
        uploadPercentage = parseInt(percentUploaded);
        if (shouldShowToast) {
          $rootScope.showToast(uploadPercentage+"%");
        }
      });

      uploadTask.$complete(function(snapshot) {
        console.log(snapshot.downloadURL);
        console.log("state_changed success",snapshot.downloadURL);
        uploadPercentage = 0;
        resolve(snapshot.downloadURL);
      });

      uploadTask.$error(function(error) {
        console.error(error);
        $rootScope.showToast('serverError');
        uploadPercentage = 0;
        reject();
      });
    });
  }

  function uploadVideo(imageData, location, name, shouldShowToast){
    if (typeof shouldShowToast === 'undefined') {
      shouldShowToast = true;
    }
    console.log('shouldShowToast', shouldShowToast);
    return $q(function(resolve, reject){
      if (typeof name === 'undefined') {
        name = '' + Date.now();
      }
      console.log("imageData",imageData);
      console.log("location",location);
      window.resolveLocalFileSystemURL('file://' + imageData, function(fileEntry){
        console.log('Type: ' + (typeof fileEntry));
        fileEntry.file(function(file){
          var type = 'video/mp4';
          var extension = 'mp4';
          if (file.hasOwnProperty('name') && file.hasOwnProperty('type')) {
            var type = file.type;
            switch (type.toLowerCase()) {
              case 'video/mp4':
              case 'video/h265':
              case 'video/h264':
              case 'video/h263': extension = 'mp4'; break;
              case 'video/quicktime': extension = 'mov'; break;
              case 'video/vp8': extension = 'webm'; break;
              case 'video/vp9': extension = 'webm'; break;
              default: extension = 'mp4'; type = 'video/mp4';
            }
          }
          console.log('File: ' + (typeof file) + ', ' + JSON.stringify(file));
          var fileReader = new FileReader();

          fileReader.onloadend = function(result){
            console.log('File Reader Result: ' + JSON.stringify(result));
            var arrayBuffer = result.target.result;
            var blob = new Blob([new Uint8Array(arrayBuffer)], {type: type});
            console.log("blob",blob);
            console.log("blob.size",blob.size);
            if(parseInt(blob.size) > (150 * 1000 * 1000)) return reject("maxSize");
            var school = $rootScope.allUserInfo.school;
            if($rootScope.school.name) school = $rootScope.school.name;
            var storageRef = firebase.storage().ref(location+'/'+name+'.'+extension);
            var storage = $firebaseStorage(storageRef);
            var uploadTask = storage.$put(blob,{contentType:type,school:school});

            return uploadProcess(uploadTask, shouldShowToast)
            .then(function(url){
              var date = getDate();
              console.log("date is",date);
              var monthDate = new Date().getMonth();
              var obj = {
                date: date,
                image: url,
                month: monthDate,
                type: 'video',
                filename: name,
                location: location+'/'+name+'.'+extension,
                extension: extension
              };
              resolve(obj);
            });
          };

          fileReader.onerror = function(error){
            reject(error);
          };

          fileReader.readAsArrayBuffer(file);
        }, function(error){
          console.log('File Entry Error: ' + JSON.stringify(error));
        });
      }, function(error){
        console.log('Error resolving file: ' + JSON.stringify(error));
      });
    });
  }

  function filetransfer(download_link, fp) {
    return $q(function(resolve,reject){
      var fileTransfer = new FileTransfer();
      // File download function with URL and local path
      console.log('entra filetransfer download', download_link);
      fileTransfer.download(download_link, fp,
        function(entry) {
          console.log("download complete:2 " + entry.fullPath);
          // $rootScope.imageFromLocal = fp;
          console.log("fp",fp);
          if($rootScope.isAndroid){
            window.galleryRefresh.refresh(decodeURI(fp),resolve,reject);
            resolve(fp);
          }else{
            resolve(entry);
          }
          // resolve();
        },
        function(error) {
          //Download abort errors or download failed errors
          console.log(JSON.stringify(error));
          console.log("download error source " + error.source);
          //alert("download error target " + error.target);
          //alert("upload error code" + error.code);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "download error source",
            errorCode: error.source,
            errorMessage: error.source
          });
          reject();
        }
      );
    });
  }

  function cacheFiletransfer(download_link, fp) {
    return $q(function(resolve,reject){
      var fileTransfer = new FileTransfer();
      // File download function with URL and local path
      console.log('entra filetransfer download', download_link);
      fileTransfer.download(download_link, fp,
        function(entry) {
          console.log('entry', entry);
          resolve(entry);
        },
        function(error) {
          //Download abort errors or download failed errors
          console.log(JSON.stringify(error));
          console.log("download error source " + error.source);
          //alert("download error target " + error.target);
          //alert("upload error code" + error.code);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "download error source",
            errorCode: error.source,
            errorMessage: error.source
          });
          reject();
        }
      );
    });
  }

  var storageRef = firebase.storage().ref();

  var image = {
    getUserImage: function(classId){
      return $q(function(resolve, reject) {
        if(!$rootScope.allUserInfo.id) reject();
        var storageRef = storage.ref('profile_images/'+$rootScope.allUserInfo.id+".jpg");
        storageRef.getDownloadURL().then(function(url) {
          // Insert url into an <img> tag to "download"
          console.log("loaded url getUserImage",url);
          $rootScope.allUserInfo.photo = url;
          resolve();
        }).catch(function(error) {
          console.log("error catch",error);
          reject();
          switch (error.code) {
            case 'storage/object_not_found':
              // File doesn't exist
              break;
            case 'storage/unauthorized':
              // User doesn't have permission to access the object
              break;

            case 'storage/canceled':
              // User canceled the upload
              break;
            case 'storage/unknown':
              // Unknown error occurred, inspect the server response
              break;
          }
        });
      });
    },saveImage: function(imageData, folder, name, extension, shouldShowToast){
      if(typeof extension === 'undefined') extension = "image/png";
      console.log("extension is",extension);
      var fileExtension = "png";
      if(extension === "application/pdf") fileExtension = "pdf";
      if(extension === "image/jpeg") fileExtension = "jpg";
      return $q(function(resolve, reject) {
        imageData = b64toBlob(imageData, extension);
        var school = $rootScope.allUserInfo.school;
        if($rootScope.school.name) school = $rootScope.school.name;
        var storageRef = firebase.storage().ref(folder+"/"+name+'.'+fileExtension);
        var storage = $firebaseStorage(storageRef);
        var uploadTask = storage.$put(imageData,{contentType:extension,school:school});

        return uploadProcess(uploadTask, shouldShowToast)
        .then(resolve);
      });
    }, saveImageFromBlob: function(blob, folder, name, extension, shouldShowToast) {
      if(typeof extension === 'undefined') extension = "image/png";
      console.log("extension is",extension);
      var fileExtension = "png";
      if(extension === "application/pdf") fileExtension = "pdf";
      if(extension === "image/jpeg") fileExtension = "jpg";
      return $q(function(resolve, reject) {
        var school = $rootScope.allUserInfo.school;
        if($rootScope.school && $rootScope.school.name) school = $rootScope.school.name;
        var storageRef = firebase.storage().ref(folder+"/"+name+'.'+fileExtension);
        var storage = $firebaseStorage(storageRef);
        var uploadTask = storage.$put(blob, {contentType: extension, school: school});

        return uploadProcess(uploadTask, shouldShowToast)
        .then(resolve);
      });
    },setUserImage: function(imageData){
      return $q(function(resolve, reject) {
        image.saveImage(imageData,"profile_images",$rootScope.allUserInfo.id).then(function(url){
          $rootScope.allUserInfo.photo = url;
          CommonFactory._save($rootScope.allUserInfo);
          resolve(url);
        },function(){
          reject();
        });
      });
    },setChildImage: function(imageData,student){
      console.log("child is",$rootScope.studentSelected);
      return $q(function(resolve, reject) {
        image.saveImage(imageData,"child_profile_images",$rootScope.studentSelected.id, 'image/jpeg').then(function(url){
          student.photo = url;
          CommonFactory._save(student);
          resolve(url);
        },function(){
          reject();
        });
      });
    },setInfoImage: function(imageData){
      return $q(function(resolve, reject) {
        image.saveImage(imageData,"info_approval",$rootScope.allUserInfo.school).then(function(url){
          var ref = firebase.database().ref('info_approval');
          $firebaseArray(ref).push({
            school:$rootScope.allUserInfo.school,
            url:url
          });
          resolve(url);
        },function(){
          reject();
        });
      });
    },getDatabaseImages: function(id, maxImages){
      var ref;
      if (maxImages === 'full') {
        ref = firebase.database().ref('gallery').child(id);
      } else {
        ref = firebase.database().ref('gallery').child(id).limitToLast(maxImages);
      }
      return $firebaseArray(ref);
    },uploadGalleryImage: function(imageData,classroomId, mediaType){
      var location = "gallery/"+classroomId;
      if(mediaType === 1) return uploadVideo(imageData, location);
      return $q(function(resolve, reject) {
        var date = getDate();
        console.log("date is",date);
        var monthDate = new Date().getMonth();
        image.saveImage(imageData, location, date).then(function(url){
          var obj = {
            date: date,
            image: url,
            month: monthDate,
            sender: $rootScope.allUserInfo.id,
            location: location + '/' + date + '.jpg'
          };
          resolve(obj);
        },function(e){
          console.log("error uploadGalleryImage",e);
          reject();
        });
      });
    },uploadGalleryImageCustomFolder: function(imageData, folderId, mediaType){
      var location = "galleryFolder/"+ $rootScope.allUserInfo.school + '/' + folderId;
      if(mediaType === 1) return uploadVideo(imageData, location);
      return $q(function(resolve, reject) {
        var date = getDate();
        console.log("date is",date);
        var monthDate = new Date().getMonth();
        image.saveImage(imageData, location, date).then(function(url){
          var obj = {
            date: date,
            image: url,
            month: monthDate,
            sender: $rootScope.allUserInfo.id,
            location: location + '/' + date + '.jpg'
          };
          resolve(obj);
        },function(e){
          console.log("error uploadGalleryImage",e);
          reject();
        });
      });
    },uploadCommunicationImage: function(imageData, fileType, shouldShowToast){
      return $q(function(resolve, reject) {
        var date = new Date().getTime();
        image.saveImage(imageData,"communications/"+$rootScope.allUserInfo.school,date,fileType, shouldShowToast).then(function(url){
          resolve(url);
        },function(e){
          console.log("error uploadCommunicationImage",e);
          reject();
        });
      });
    },uploadDiaryImage: function(imageData, fileType, id){
      return $q(function(resolve, reject) {
        var date = new Date().getTime();
        image.saveImage(imageData,"diary/"+id,date,fileType).then(function(url){
          resolve({url: url, date: date});
        },function(e){
          console.log("error uploadDiaryImage",e);
          reject();
        });
      });
    },uploadDocument: function(imageData, fileType){
      return $q(function(resolve, reject) {
        var date = new Date().getTime();
        image.saveImage(imageData,"documents/"+$rootScope.allUserInfo.school,date,fileType).then(function(url){
          resolve(url);
        },function(e){
          console.log("error uploadCommunicationImage",e);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "Error upload document",
            errorCode:e,
            errorMessage:e
          });
          reject();
        });
      });
    }, uploadDocumentDefault: function(imageData, fileType, folder, shouldShowToast){
      return $q(function(resolve, reject) {
        if (typeof shouldShowToast === 'undefined') {
          shouldShowToast = true;
        }
        var date = new Date().getTime();
        image.saveImage(imageData, folder + "/" + $rootScope.allUserInfo.school, date, fileType, shouldShowToast).then(function(url){
          resolve(url);
        },function(e){
          console.log("error uploadCommunicationImage",e);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "Error upload document",
            errorCode:e,
            errorMessage:e
          });
          reject();
        });
      });
    },deleteImage: function(classroomId, id) {
      return firebase.database().ref('gallery').child(classroomId).child(id).remove();
    },getCustomFolders: function(schoolId) {
      loadedSchoolId = schoolId;
      return $firebaseArray(firebase.database().ref('galleryFolders/folders/' + schoolId));
    },getCustomFolderImages: function(folderId, maxImages) {
      if (typeof maxImages === 'undefined') {
        maxImages = 'full';
      }
      var customRef = 'galleryFolders/images/' + loadedSchoolId + '/' + folderId;
      var ref;
      if (maxImages === 'full') {
        ref = firebase.database().ref(customRef);
      } else {
        ref = firebase.database().ref(customRef).limitToLast(maxImages);
      }
      return $firebaseArray(ref);
    },deleteCustomFolderImage: function(folderId, imageId) {
      return firebase.database().ref('galleryFolders/images/' + loadedSchoolId + '/' + folderId + '/' + imageId).remove();
    }, displayGalleryImage: function(){


      /*function displayImageByFileURL(fileEntry) {
          var elem = document.getElementById('imageFile');
          elem.src = fileEntry.toURL();
      } */
    },downloadImgToGallery: function(download_link, folderName, fileName){
      console.log("download_link",download_link);
      console.log("folderName",folderName);
      console.log("fileName",fileName);
      function gotFileEntry(fileEntry) {
          console.log("succeees"+JSON.stringify(fileEntry));
      };
      function onGetDirectorySuccess(dir) {
        console.log("entra 3");
        // dir.getFile("pruebaFilename", {
        //     create: true,
        //     exclusive: false
        // }, gotFileEntry, onError);
        console.log("dir"+JSON.stringify(dir));
        var ext = download_link.substr(download_link.lastIndexOf('.') + 1); //Get extension of URL
        if(ext.indexOf('?') !== -1) ext = ext.substr(0,ext.indexOf('?'));
        console.log("dir.nativeURL"+dir.nativeURL + folderName + "/" + fileName + "." + ext);
        filetransfer(download_link, dir.nativeURL + folderName + "/" + fileName + "." + ext)
        .then(function(){
          console.log("imagen descargada!");
          alert($translate.instant('image_downloaded',{'folder':folderName}));
        },function(error){
          console.log("error en descarga de imagen"+error);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "Error al descargar imagen",
            errorCode: 'Error',
            errorMessage: error
          });
          alert($translate.instant('image_download_otherError'));
        });
      };
      function onFileSystemSuccess(fileSystem){
        console.log("entra 2");
        var entry = "";
        if ($rootScope.isAndroid) {
            entry = fileSystem;
        } else {
            entry = fileSystem.root;
        }
        console.log("entra 2"+JSON.stringify(entry));
        entry.getDirectory(configuration.appName, {
            create: true,
            exclusive: false
        }, onGetDirectorySuccess, onError);
      };
      function onError(error){
        CommonFactory.saveLog({
          user: $rootScope.allUserInfo.id,
          error: "Error al acceder a la carpeta para descargar imagen",
          errorCode: 'Error',
          errorMessage: error
        });
        alert($translate.instant('image_download_folderError'));
      };
      if($rootScope.isAndroid){
        window.resolveLocalFileSystemURL(cordova.file.externalRootDirectory, onFileSystemSuccess, onError);
      }else{
        var ext = download_link.substr(download_link.lastIndexOf('.') + 1); //Get extension of URL
        if(ext.indexOf('?') !== -1) ext = ext.substr(0,ext.indexOf('?'));
        var targetPath = decodeURIComponent(cordova.file.documentsDirectory+fileName + '.' + ext);
        filetransfer(download_link, targetPath)
        .then(function(result){
          // console.log("imagen descargada!",result.toUrl());
          window.plugins.socialsharing.share(null, null, decodeURIComponent(result.toURL()));
        },function(error){
          console.log("error en descarga de imagen"+error);
          alert($translate.instant('serverError'));
        });

      }
    }, downloadDocumentToLocal: function(download_link, fileName) {
      return $q(function (resolve, reject) {
        console.log('download_link', download_link, fileName);
        var directory;
        if ($rootScope.isAndroid) {
          // directory = cordova.file.externalCacheDirectory;
          resolve(download_link);
          return;
        } else {
          directory = cordova.file.cacheDirectory;
        }
        var ext = download_link.substr(download_link.lastIndexOf('.') + 1); //Get extension of URL
        if(ext.indexOf('?') !== -1) ext = ext.substr(0,ext.indexOf('?'));
        var targetPath = decodeURIComponent(directory + fileName + '.' + ext);
        cacheFiletransfer(download_link, targetPath)
        .then(function(result){
          console.log("documento descargado!", result);
          resolve(result.toURL());
        },function(error){
          console.log("error en descarga de imagen"+error);
          reject();
        });
      });
    }, shareImage: function(imageUrl) {
      $rootScope.show();
      convertUrlToBase64(imageUrl)
      .then(function(imageBase64) {
        // this is the complete list of currently supported params you can pass to the plugin (all optional)
        var options = {
          // message: 'share this', // not supported on some apps (Facebook, Instagram)
          // subject: 'the subject', // fi. for email
          // files: [encodeURI(imageUrl)], // an array of filenames either locally or remotely
          files: [
            imageBase64
          ], // an array of filenames either locally or remotely
          chooserTitle: $translate.instant('share_title') // Android only, you can override the default share sheet title,
        };
  
        var onShareSuccess = function(result) {
          $rootScope.hide();
          console.log("Share completed? " + result.completed); // On Android apps mostly return false even while it's true
          console.log("Shared to app: " + result.app); // On Android result.app since plugin version 5.4.0 this is no longer empty. On iOS it's empty when sharing is cancelled (result.completed=false)
        };
  
        var onShareError = function(msg) {
          $rootScope.hide();
          alert($translate.instant('share_error'));
          console.log("Sharing failed with message: " + msg);
          CommonFactory.saveLog({
            user: $rootScope.allUserInfo.id,
            error: "Error share image",
            errorCode: 'Error',
            errorMessage: msg
          });
        };
  
        window.plugins.socialsharing.shareWithOptions(options, onShareSuccess, onShareError);
      });
    },savePhotoFromUrlToInternal: function(url, folderName, fileName){
      return $q(function(resolve,reject){
        window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, fileSystemSuccess, fileSystemFail);

        function fileSystemSuccess(fileSystem) {
            // var download_link = encodeURI(url);
            var download_link = url;
            console.log("download_link",download_link);
            var ext = download_link.substr(download_link.lastIndexOf('.') + 1); //Get extension of URL
            if(ext.indexOf('?') !== -1) ext = ext.substr(0,ext.indexOf('?'));

            console.log("fileSystem.root",fileSystem.root);
            // var directoryEntry = fileSystem.root; // to get root path of directory
            var directoryEntry = fileSystem.root; // to get root path of directory
            // console.log("directoryEntry is",directoryEntry);
            directoryEntry.getDirectory(folderName, { create: true, exclusive: false }, onDirectorySuccess, onDirectoryFail); // creating folder in sdcard
            // var rootdir = cordova.file.dataDirectory;
            var rootdir = directoryEntry;
            console.log("rootdir"+JSON.stringify(rootdir));
            // var fp = rootdir.fullPath; // Returns Fulpath of local directory
            // var fp = rootdir.nativeURL;
            var fp = rootdir.toURL();
            console.log("fp",fp);

            fp = fp +  folderName + "/" + fileName + "." + ext; // fullpath and name of the file which we want to give
            console.log("fp",fp);
            // download function call
            filetransfer(download_link, fp).then(function(){
              resolve(fp);
            },function(){
              reject();
            });
        }

        function onDirectorySuccess(parent) {
            // Directory created successfuly
            console.log("directory created!");
        }

        function onDirectoryFail(error) {
            //Error while creating directory
            console.log("Unable to create new directory: " + error.code);
        }

        function fileSystemFail(evt) {
          //Unable to access file system
          console.log(evt.target.error.code);
       }
      });
    }, uploadVideo: function(imageData, location, name) {
      return uploadVideo(imageData, location, name, false);
    }, getPercentageProgress: function() {
      return uploadPercentage;
    }, getDocumentUrl: function(item) {
      return $q(function(resolve, reject) {
        console.log('entra a getDocumentUrl', item);
        if (!item.location) {
          resolve(item);
          return;
        }
        $firebaseStorage(firebase.storage().ref(item.location)).$getDownloadURL().then(function(url) {
          console.log('url', url);
          item.url = url;
          resolve(item);
        }, function(error) {
          console.log('error', error);
          resolve(item);
        });
      });
    }, openDocument: function(item) {
      // document: {name: string, url: string}
      return $q(function(resolve, reject) {
        image.getDocumentUrl(item)
        .then(function(document) {
          var name = document.name.toLowerCase().split(' ');
          name = name[0].replace(/á/g, 'a').replace(/à/g, 'a').replace(/é/g, 'e')
          .replace(/è/g, 'e').replace(/í/g, 'i').replace(/ó/g, 'o')
          .replace(/ò/g, 'o').replace(/ú/g, 'u').replace(/ /g, '_')
          .replace('ñ', 'n').replace('ç', 'c').replace('€', 'e').replace('$', 'usd');
          image.downloadDocumentToLocal(document.url, encodeURI(name))
          .then(function(url) {
            PreviewAnyFile.preview(url,
            function(win) {
              if (win == "SUCCESS") {
                console.log('success');
              } else if (win == "CLOSING") {
                console.log('closing');
              } else {
                console.log('error');
              }
              resolve();
            }, function(err){
              console.log('err', err);
              reject();
            });
            resolve();
          }, function(err){
            console.log('err', err);
            reject();
          });
        });
      });
    }, getVideoSize: function(videoUrl) {
      return $q(function(resolve, reject) {
        window.resolveLocalFileSystemURL('file://' + videoUrl, function(fileEntry) {
          fileEntry.file(function(file){
            console.log('file is', file);
            var fileInformation = {
              size: false,
              type: false
            };
            try {
              if (!!file.size) {
                fileInformation.size = file.size / 1024;
              }
              if (!!file.type) {
                fileInformation.type = file.type;
              }
              VideoEditor.getVideoInfo(
                /*
                  audioMediaType: "audio/mp4a-latm"
                  bitrate: 42340249
                  bitrate: 8110057
                  duration: 19.885
                  height: 2160
                  orientation: "landscape"
                  size: 105240686
                  videoMediaType: "video/avc"
                  width: 3840
                */
                function (info) {
                  console.log('video info', info);
                  info.type = fileInformation.type;
                  resolve(info);
                },
                function (error) {
                  console.log('error', error);
                  resolve(fileInformation);
                },
                {
                    fileUri: 'file://' + videoUrl
                }
              );
            } catch (error) {
              resolve(false);
            }
          }, function() {
            resolve(false);
          });
        }, function() {
          resolve(false);
        });
      });
    }, getAndAskStoragePermissions: function(type) {
      return $q(function(resolve, reject) {
        try {
          var permission;
          if (type === 'read') {
            permission = cordova.plugins.permissions.READ_EXTERNAL_STORAGE;
          } else {
            permission = cordova.plugins.permissions.WRITE_EXTERNAL_STORAGE;
          }
          cordova.plugins.permissions.checkPermission(permission, function(result) {
            if (!result.hasPermission) {
              cordova.plugins.permissions.requestPermission(permission, function(requestResult) {
                if (requestResult.hasPermission) {
                  resolve();
                } else {
                  reject();
                }
              }, resolve);
            } else {
              resolve();
            }
          }, resolve);
        } catch (error) {
          console.log(error);
          resolve();
        }
      });
    }, multipleImages: {
      getCompressedImages: function(medias) {
        return $q(function (resolve, reject) {
          thumbnails = [];
          var promises = [];
          for (var i = 0; i < medias.length; i++) {
            promises.push(image.multipleImages.compressImage(medias[i]));
          }
          $q.all(promises)
          .then(function (result) {
            resolve(result);
          });
        });
      }, getCompressedVideos: function(medias) {
        return $q(function (resolve, reject) {
          thumbnails = [];
          var promises = [];
          for (var i = 0; i < medias.length; i++) {
            promises.push(image.multipleImages.compressVideo(medias[i]));
          }
          $q.all(promises)
          .then(function (result) {
            resolve(result);
          });
        });
      }, compressImage: function(media) {
        return $q(function (resolve, reject) {
          media.quality = 50;
          MediaPicker.compressImage(media, function(imageCompressed) {
            MediaPicker.fileToBlob(imageCompressed.path, function(data) {
              resolve({
                path: imageCompressed.path,
                file: data,
                index: imageCompressed.index
              });
            });
          }, function(e) {
            console.log(e);
            return resolve();
          });
        });
      }, compressVideo: function(media) {
        return $q(function (resolve, reject) {
          console.log('media', media);
          switch (true) {
            case media.size > 50874368: media.quality = 20; break;
            case media.size > 10874368: media.quality = 30; break;
            case media.size > 5874368: media.quality = 50; break;
            case media.size > 2874368: media.quality = 70; break;
            case media.size <= 2874368: media.quality = 80; break;
            default: media.quality = 50;
          }
          VideoEditor.transcodeVideo(
            function (info) {
              console.log("success transcoding video", info);
              resolve(info);
            }, // success cb
            function (error) {
              // alert($translate.instant('gallery_errorVideo'));
              console.log("error transcoding video", error);
              resolve(media.uri);
            }, // error cb
            {
              fileUri: media.uri, // the path to the video on the device
              outputFileName: 'childcareon-diaryThumb_'+ new Date().getTime(), // the file name for the transcoded video
              outputFileType: VideoEditorOptions.OutputFileType.MPEG4, // android is always mp4
              optimizeForNetworkUse: VideoEditorOptions.OptimizeForNetworkUse.YES, // ios only
              saveToLibrary: false, // optional, defaults to true
              deleteInputFile: false, // optional (android only), defaults to false
              maintainAspectRatio: true, // optional (ios only), defaults to true
              width: 640, // optional, see note below on width and height
              height: 640, // optional, see notes below on width and height
              videoBitrate: 8000000, // optional, bitrate in bits, defaults to 1 megabit (1000000)
              fps: 25, // optional (android only), defaults to 24
              // audioChannels: 2, // optional (ios only), number of audio channels, defaults to 2
              // audioSampleRate: 44100, // optional (ios only), sample rate for the audio, defaults to 44100
              // audioBitrate: 128000, // optional (ios only), audio bitrate for the video in bits, defaults to 128 kilobits (128000)
              progress: function(info) {console.log("progress", info);} // info will be a number from 0 to 100
            }
          );
        });
      }, uploadVideos: function (videos, serverLocation) {
        return $q(function (resolve, reject) {
          var promises = [];
          var date = new Date().getTime().toString();
          date = date.slice(0, date.length -1);
          var counter = 0;
          for (var i=0; i < videos.length; i++) {
            var dateImage = date + counter.toString();
            counter++;
            // promises.push(image.multipleImages.uploadImage(images[i].file, location, parseInt(dateImage)));
            promises.push(uploadVideo(videos[i], serverLocation, dateImage));
          }
          $q.all(promises)
          .then(function (result) {
            resolve(result);
          }, function(error) {
            console.log('error uploading images', error);
            reject();
          });
        });
      }, uploadImages: function (images, location) {
        return $q(function (resolve, reject) {
          var promises = [];
          var date = new Date().getTime().toString();
          date = date.slice(0, date.length -1);
          var counter = 0;
          for (var i=0; i < images.length; i++) {
            var dateImage = date + counter.toString();
            counter++;
            promises.push(image.multipleImages.uploadImage(images[i].file, location, parseInt(dateImage)));
          }
          $q.all(promises)
          .then(function (result) {
            resolve(result);
          }, function(error) {
            console.log('error uploading images', error);
            reject();
          });
        });
      }, uploadImage(imageToUpload, location, date) {
        return $q(function (resolve, reject) {
          image.saveImageFromBlob(imageToUpload, location, date, 'image/jpeg', false)
          .then(function (url) {
            resolve({
              date: date,
              image: url,
              location: location + '/' + date + '.jpg'
            });
          }, function(error) {
            console.log('error uploading image', error);
            reject();
          });
        });
      }
    }
  };
  return image;
});
